 ;+
;Procedure: THM_LOAD_PROC_ARG
;
;Purpose:  Generic argument validation routine for THEMIS Data File Loading 
;   routines, meant to be called by
;   instrument-specific thm_load procedures.
;
;keywords:
;  post_process_proc: name of procedure to call after cdf2tplot is called
;                     will be called w/ keywords sname, dt (datatype), lvl,
;                     and _extra.
;  relpath_funct: name of routine to call in place of file_dailynames
;                 may simply be a wrapper.
;                 will be called w/ keywords sname, dt (datatype), lvl,
;                 and _extra.
;  cdf_to_tplot: user-supplied procedure to override cdf2tplot
;  sname  = site or probe name. The default is 'all',
;  type_sname = string, set to 'probe' or 'site'
;  vsnames = space-separated list of valid probes/sites
;  datatype = The type of data to be loaded, for this case, there is only
;          one option, the default value of 'fgm', so this is a
;          placeholder should there be more that one data type. 'all'
;          can be passed in also, to get all variables.
;  vdatatypes = space-separated list of valid data types
;  TRANGE= (Optional) Time range of interest  (2 element array), if
;          this is not set, the default is to prompt the user. Note
;          that if the input time range is not a full day, a full
;          day's data is loaded
;  level = the level of the data, the default is 'l1', or level-1
;          data. A string (e.g., 'l2') or an integer can be used. 'all'
;          can be passed in also, to get all levels.
;  vlevels
;  CDF_DATA: named variable in which to return cdf data structure: only works
;          for a single spacecraft and datafile name.
;  VARNAMES: names of variables to load from cdf: default is all.
;  /GET_SUPPORT_DATA: load support_data variables as well as data variables
;                      into tplot variables.
;  /DOWNLOADONLY: download file but don't read it.
;  /valid_names, if set, then this routine will return the valid probe, datatype
;          and/or level options in named variables supplied as
;          arguments to the corresponding keywords.
;  files   named varible for output of pathnames of local files.
;  /VERBOSE  set to output some useful info
;
;Notes:
;  This routine is (should be) platform independent.
;
; $LastChangedBy: jimm $
; $LastChangedDate: 2007-06-05 11:58:33 -0700 (Tue, 05 Jun 2007) $
; $LastChangedRevision: 740 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/branches/QA/idl/themis/common/thm_load_proc_arg.pro $
;-
pro thm_load_proc_arg,sname=sname, datatype=datatype, $
                      level=level, verbose=verbose, no_download=no_download, $
                      valid_names = valid_names, $
                      type_sname=type_sname, $
                      vsnames=vsnames, vdatatypes=vdatatypes, vlevels=vlevels, $
                      deflevel=deflevel, $
                      get_cdf_data=get_cdf_data, $
                      osname=snames, odt=dts, olvl=lvls, my_themis=my_themis, $
                      _ref_extra = _extra

  thm_init
; If verbose keyword is defined, override !themis.verbose
  vb = size(verbose, /type) ne 0 ? verbose : !themis.verbose

; Valid sname names
  vsnames = strsplit(vsnames, ' ', /extract)
; Valid data names
  vdatatypes = strsplit(vdatatypes, ' ', /extract) ;for completeness
;valid data levels
  vlevels = strsplit(vlevels, ' ', /extract)
;If valid_names is set, return the options
  if keyword_set(valid_names) then begin
    sname = vsnames
    message, /info, string(strjoin(sname, ','), format = '( "Valid '+type_sname+'s:",X,A,".")')
    datatype = vdatatypes
    message, /info, string(strjoin(datatype, ','), format = '( "Valid datatypes:",X,A,".")')
    level = vlevels
    message, /info, string(strjoin(level, ','), format = '( "Valid levels:",X,A,".")')
    return
  endif
;parse out snames
  if not keyword_set(sname) then snames = vsnames $
  else snames = thm_check_valid_name(strlowcase(sname), vsnames, /include_all)
  if not keyword_set(snames) then return
  if keyword_set(vb) then printdat, snames, /value, type_sname+'s'
;datatype
  if not keyword_set(datatype) then dts = vdatatypes $
  else dts = thm_check_valid_name(strlowcase(datatype), vdatatypes, /include_all)
  if not keyword_set(dts) then return
  if keyword_set(vb) then printdat, dts, /value, 'Datatypes'
;data level
  if keyword_set(deflevel) then lvl = deflevel else lvl = 'l1'
  if n_elements(level) gt 0 then begin
    if size(level, /type) Eq 7 then begin
      If(level[0] Ne '') Then lvl = strcompress(strlowcase(level), /remove_all)
    endif else lvl = 'l'+strcompress(string(fix(level)), /remove_all)
  endif
  lvls = thm_check_valid_name(strlowcase(lvl), vlevels, /include_all)
  if not keyword_set(lvls) then return
  if keyword_set(vb) then printdat, lvls, /value, 'Level'

  if vb ge 7 then printdat,!themis

  nlvls = n_elements(lvls)
  ndts = n_elements(dts)
  nsnames = n_elements(snames)

  my_themis = !themis
  my_themis.no_download = !themis.no_download || keyword_set(no_download) 
  my_themis.verbose=vb

end
