;+
;NAME:
;thm_ui_load_data_fn
;PURPOSE:
;A widget interface to load CDF data for whatever instrument
;CALLING SEQUENCE:
;varnames = thm_ui_load_data_fn(st_time, en_time, $
;                              dtype = dtype, $
;                              station = station, $
;                              astation = astation, $
;                              probe = probe)
;INPUT:
;st_time, en_time = start and end times in seconds from
;                   1-jan-1970 0:00
;KEYWORDS:
;dtype, the type of data, a string, of form
;       'instrument/datatype/datalevel', the default is 'gmag/mag/l2'
;station, the ground station of the gmag data, default is '*', for all
;astation, the ground station of the asi data, default is '*', for all
;probe = one or more of ['a','b','c','d','e'], default is ['a','b','c','d','e']
;OUTPUT:
;varnames = an array of tplot variable names, to pass into tplot
;HISTORY:
; 22-sep-2006, jmm, jimm@ssl.berkeley.edu
; 23-oct-2006, jmm, changed to call cdf2tplot
; 30-oct-2006, jmm, changed again, to call load_thg_mag, also added 
;                   dtype and station as inputs
; 13-nov-2006, jmm, changed the argument list, now the output is a
;                   data id string, added a check for start time > end
;                   time
; 08-dec-2006, krb, changed load_thg_mag to thm_load_gmag
; 13-dec-2006, jmm, Added call to cdf2tplot for data for which there
;                   is no load program...
; 14-dec-2006, jmm, returns an array of variable names and
;                   subscripts, which should be easier to read..
; 15-dec-2006, jmm, Added calls to thm_load_fgm, fit, sst, efi,
;                   ask,probe can be an array
; 16-jan-2007, jmm, Add all of the thm_load routines
; 5-feb-2007, jmm, update calls to load routines
; 10-apr-2007, jmm, removed history tracking, which is now handled in
;                   thm_gui_event, also changed dtype, station
;                   astation and probe to keywords
; 1-may-2007, jmm, New version, new argument list,etc....
; 10-may-2007, jmm, New version, new argument list,etc... that last
;                   one lasted a long time, didn't it
;
;$LastChangedBy$
;$LastChangedDate$
;$LastChangedRevision$
;$URL$
;
;-
Function thm_ui_load_data_fn, st_time, en_time, $
                              dtype = dtype0, $
                              station = station0, $
                              astation = astation0, $
                              probe = probe0, $
                              progobj = progobj, $
                              _extra = _extra
  @tplot_com
  otp = -1
  data_ss = -1
;database
  thm_init                      ;this should work
  If(n_elements(dtype0) Eq 0) Then dtype = 'gmag/mag/l2' $
  Else dtype = strlowcase(strcompress(dtype0, /remove_all))
  ndtype = n_elements(dtype)
;get the instrument type datatype and dlevel
  instr = strarr(ndtype) & iname = instr & dlvl = instr
  For j = 0, ndtype-1 Do Begin
    ppp = strsplit(dtype[j], '/', /extract)
    instr[j] = ppp[0]
    iname[j] = ppp[1]
    dlvl[j] = ppp[2]
  Endfor
  If(n_elements(station0) Eq 0) Then station = '*' $
  Else station = strlowcase(strtrim(station0, 2))
  If(n_elements(astation0) Eq 0) Then astation = '*' $
  Else astation = strlowcase(strtrim(astation0, 2))
  If(n_elements(probe0) Eq 0) Then probe = ['a', 'b', 'c', 'd', 'e'] $
  Else probe = strlowcase(strtrim(probe0, 2))
;Get the info for the data that has been loaded already
  If(is_struct(data_quants)) Then Begin
    tx0 = time_string(data_quants.trange)
    didx0 = data_quants.name+':'+tx0[0, *]+' To '+tx0[1, *]
  Endif Else didx0 = -1
  If(en_time Le st_time) Then Begin
    message, 'End time is less than Start time? No data Loaded'
  Endif
;people are inconsistent in using the trange keyword, so set a
;timespan...:
  tt = [st_time, en_time]
  t1 = str2time(strmid(time_string(tt[0]), 0, 10))
  t2 = str2time(strmid(time_string(tt[1]-1.), 0, 10))
  ndays = 1+fix((t2-t1)/(24.*3600.))
  timespan, t1, ndays
;GMAG
  gmag_ss = where(instr Eq 'gmag')
  If(gmag_ss[0] Ne -1) Then Begin
    thm_load_gmag, site = station, trange = [st_time, en_time], progobj = progobj
  Endif
;ASI
  asi_ss = where(instr Eq 'asi')
  If(asi_ss[0] Ne -1) Then Begin
    thm_load_asi, site = astation, trange = [st_time, en_time], $
      datatype = iname[asi_ss], progobj = progobj
  Endif
;ASK
  ask_ss = where(instr Eq 'ask')
  If(ask_ss[0] Ne -1) Then Begin
    thm_load_ask, trange = [st_time, en_time], datatype = iname[ask_ss], progobj = progobj
  Endif
;EFI
  efi_ss = where(instr Eq 'efi')
  If(efi_ss[0] Ne -1) Then Begin
    thm_load_efi, probe = probe, datatype = iname[efi_ss], $
      /get_support_data, trange = [st_time, en_time], progobj = progobj
  Endif
;FBK
  fbk_ss = where(instr Eq 'fbk')
  If(fbk_ss[0] Ne -1) Then Begin
    thm_load_fbk, probe = probe, datatype = iname[fbk_ss], $
      /get_support_data, trange = [st_time, en_time], progobj = progobj
  Endif
;FFT
  fft_ss = where(instr Eq 'fft')
  If(fft_ss[0] Ne -1) Then Begin
    thm_load_fft, probe = probe, datatype = iname[fft_ss], type = 'raw', $
      /get_support_data, trange = [st_time, en_time], progobj = progobj
  Endif
;FGM
  ss = where(instr Eq 'fgm')
  If(ss[0] Ne -1) Then Begin
    u_lev = uniq(dlvl[ss])
    lvls = dlvl[ss[u_lev]]
;load state data, if not there
    For j = 0, n_elements(probe)-1 Do Begin
      thm_ui_check4spin, 'th'+probe[j]+'_fgm_dummy', vx1, vx1, h1, $
        probe_in = probe[j], trange = [st_time, en_time], $
        progobj = progobj
    Endfor
    For j = 0, n_elements(lvls)-1 Do Begin
      ssj = where(dlvl[ss] Eq lvls[j])
      If(ssj[0] Ne -1) Then Begin
        thm_load_fgm, probe = probe, datatype = iname[ss[ssj]], $
          level = lvls[j], /get_support_data, trange = [st_time, en_time], $
          progobj = progobj
      Endif
    Endfor
  Endif
;FIT
  fit_ss = where(instr Eq 'fit')
  If(fit_ss[0] Ne -1) Then Begin
    thm_load_fit, probe = probe, datatype = iname[fit_ss], $
      /get_support_data, trange = [st_time, en_time], progobj = progobj
  Endif
;SCM
  scm_ss = where(instr Eq 'scm')
  If(scm_ss[0] Ne -1) Then Begin
    thm_load_scm, probe = probe, datatype = iname[scm_ss], $
      /get_support_data, trange = [st_time, en_time], progobj = progobj
  Endif
;MOM
  mom_ss = where(instr Eq 'mom')
  If(mom_ss[0] Ne -1) Then Begin
    thm_load_mom, probe = probe, datatype = iname[mom_ss], $
      trange = [st_time, en_time], progobj = progobj
  Endif
;SST
  sst_ss = where(instr Eq 'sst')
  If(sst_ss[0] Ne -1) Then Begin
    thm_load_sst, probe = probe, datatype = iname[sst_ss], $
      /get_support_data, trange = [st_time, en_time], progobj = progobj
  Endif
;ESA
  esa_ss = where(instr Eq 'esa_pkt')
  If(esa_ss[0] Ne -1) Then Begin
;bizzarre esa problem
    ddd = strcompress(/remove_all, string(fix(iname[esa_ss])))
    thm_load_esa_pkt, probe = probe, datatype = ddd, $
      /get_support_data, trange = [st_time, en_time]
  Endif
;STATE
  state_ss = where(instr Eq 'state')
  If(state_ss[0] Ne -1) Then Begin
    thm_load_state, probe = probe, datatype = iname[state_ss], $
      /get_support_data, trange = [st_time, en_time], progobj = progobj
  Endif

;What are the new structure elements?
  If(is_struct(data_quants)) Then Begin
    ndq = n_elements(data_quants)
    If(ndq Gt 1) Then Begin
      If(is_string(didx0)) Then Begin
        tx = time_string(data_quants.trange)
        didx = data_quants.name+':'+tx[0, *]+' To '+tx[1, *]
        new_flag = bytarr(ndq)
        For j = 0, ndq-1 Do Begin
          fff = where(didx0 Eq didx[j], nfff)
          If(nfff Eq 0) Then new_flag[j] = 1
        Endfor
        data_ss = where(new_flag Eq 1)
      Endif Else data_ss = 1+lindgen(n_elements(data_quants)-1)
;Here restrict to the input time range
;    thm_ui_only_trange, st_time, en_time, data_ss
;Here get the tplot variable names
      If(data_ss[0] Ne -1) Then Begin
        tplotvars = tnames()
        otp = tplotvars[data_ss-1]
      Endif Else otp = -1
    Endif Else Begin
      data_ss = -1
      otp = -1
    Endelse
  Endif Else Begin
    data_ss = -1
    otp = -1
  Endelse
  Return, otp
End
