;+
;FUNCTION:	j_3d_new(dat,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
;INPUT:	
;	dat:	structure,	2d data structure filled by get_eesa_surv, get_eesa_burst, etc.
;KEYWORDS
;	ENERGY:	fltarr(2),	optional, min,max energy range for integration
;	ERANGE:	fltarr(2),	optional, min,max energy bin numbers for integration
;	EBINS:	bytarr(na),	optional, energy bins array for integration
;					0,1=exclude,include,  
;					na = dat.nenergy
;	ANGLE:	fltarr(2,2),	optional, angle range for integration
;				theta min,max (0,0),(1,0) -90<theta<90 
;				phi   min,max (0,1),(1,1)   0<phi<360 
;	ARANGE:	fltarr(2),	optional, min,max angle bin numbers for integration
;	BINS:	bytarr(nb),	optional, angle bins array for integration
;					0,1=exclude,include,  
;					nb = dat.ntheta
;	BINS:	bytarr(na,nb),	optional, energy/angle bins array for integration
;					0,1=exclude,include
;PURPOSE:
;	Returns the flux, [Jx,Jy,Jz], 1/(cm^2-s) 
;NOTES:	
;	Function normally called by "get_3dt" or "get_2dt" to
;	generate time series data for "tplot.pro".
;
;CREATED BY:
;	J.McFadden	00-2-24
;LAST MODIFICATION:
;-
function j_3d_new,dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins

flux3dx = 0.
flux3dy = 0.
flux3dz = 0.

if dat2.valid eq 0 then begin
  print,'Invalid Data'
  return, [flux3dx,flux3dy,flux3dz]
endif

dat = conv_units(dat2,"df")		; Use distribution function
na = dat.nenergy
nb = dat.nbins
	
ebins2=replicate(1b,na)
if keyword_set(en) then begin
	ebins2(*)=0
	er2=[energy_to_ebin(dat,en)]
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(er) then begin
	ebins2(*)=0
	er2=er
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(ebins) then ebins2=ebins

bins2=replicate(1b,nb)
if keyword_set(an) then begin
	if ndimen(an) ne 2 then begin
		print,'Error - angle keyword must be (2,2)'
	endif else begin
		bins2=angle_to_bins(dat,an)
	endelse
endif
if keyword_set(ar) then begin
	bins2(*)=0
	if ar(0) gt ar(1) then begin
		bins2(ar(0):nb-1)=1
		bins2(0:ar(1))=1
	endif else begin
		bins2(ar(0):ar(1))=1
	endelse
endif
if keyword_set(bins) then bins2=bins

if ndimen(bins2) ne 2 then bins2=ebins2#bins2

data = dat.data*bins2
energy = dat.energy
denergy = dat.denergy
theta = dat.theta/!radeg
phi = dat.phi/!radeg
dtheta = dat.dtheta/!radeg
dphi = dat.dphi/!radeg
domega = dat.domega
	if ndimen(domega) eq 1 then domega=replicate(1.,dat.nenergy)#domega
mass = dat.mass
Const = 2./mass/mass*1e5
charge=1.
value=0 & str_element,dat,'charge',value
if value ne 0 then charge=dat.charge		
if ((value eq 0) and (dat.mass lt 0.00010438871)) then charge=-1.		; this line works for Wind which does not have dat.charge
value=0 & str_element,dat,'sc_pot',value
if value gt 0 or value lt 0 then energy=energy+(charge*dat.sc_pot/abs(charge))>0.		; energy/charge analyzer

solid_angle_corr=4.*!pi/total(domega(0,*))	; this should be correct in the structure
if (solid_angle_corr lt .99 or solid_angle_corr gt 1.01) and max(theta) gt 1.2 then print,'Error in dat.domega'   
solid_angle_corr=1.

flux3dx = Const*total(denergy*(energy)*data*(dtheta/2.+cos(2*theta)*sin(dtheta)/2.)*(2.*sin(dphi/2.)*cos(phi)))
flux3dy = Const*total(denergy*(energy)*data*(dtheta/2.+cos(2*theta)*sin(dtheta)/2.)*(2.*sin(dphi/2.)*sin(phi)))
flux3dz = Const*total(denergy*(energy)*data*(2.*sin(theta)*cos(theta)*sin(dtheta/2.)*cos(dtheta/2.))*dphi)

; units are 1/cm^2-s

return, [flux3dx,flux3dy,flux3dz]
end
;+
;FUNCTION:	je_3d_new(dat,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
;INPUT:	
;	dat:	structure,	3d data structure filled by get_eesa_surv, get_eesa_burst, etc.
;KEYWORDS
;	ENERGY:	fltarr(2),	optional, min,max energy range for integration
;	ERANGE:	fltarr(2),	optional, min,max energy bin numbers for integration
;	EBINS:	bytarr(na),	optional, energy bins array for integration
;					0,1=exclude,include,  
;					na = dat.nenergy
;	ANGLE:	fltarr(2,2),	optional, angle range for integration
;				theta min,max (0,0),(1,0) -90<theta<90 
;				phi   min,max (0,1),(1,1)   0<phi<360 
;	ARANGE:	fltarr(2),	optional, min,max angle bin numbers for integration
;	BINS:	bytarr(nb),	optional, angle bins array for integration
;					0,1=exclude,include,  
;					nb = dat.ntheta
;	BINS:	bytarr(na,nb),	optional, energy/angle bins array for integration
;					0,1=exclude,include
;PURPOSE:
;	Returns the energy flux, [JEx,JEy,JEz], 1/(cm^2-s) 
;NOTES:	
;	Function normally called by "get_3dt" or "get_2dt" to
;	generate time series data for "tplot.pro".
;
;CREATED BY:
;	J.McFadden	02-9-13
;LAST MODIFICATION:
;-
function je_3d_new,dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins

eflux3dx = 0.
eflux3dy = 0.
eflux3dz = 0.

if dat2.valid eq 0 then begin
  print,'Invalid Data'
  return, [eflux3dx,eflux3dy,eflux3dz]
endif

dat = conv_units(dat2,"df")		; Use distribution function
na = dat.nenergy
nb = dat.nbins
	
ebins2=replicate(1b,na)
if keyword_set(en) then begin
	ebins2(*)=0
	er2=[energy_to_ebin(dat,en)]
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(er) then begin
	ebins2(*)=0
	er2=er
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(ebins) then ebins2=ebins

bins2=replicate(1b,nb)
if keyword_set(an) then begin
	if ndimen(an) ne 2 then begin
		print,'Error - angle keyword must be (2,2)'
	endif else begin
		bins2=angle_to_bins(dat,an)
	endelse
endif
if keyword_set(ar) then begin
	bins2(*)=0
	if ar(0) gt ar(1) then begin
		bins2(ar(0):nb-1)=1
		bins2(0:ar(1))=1
	endif else begin
		bins2(ar(0):ar(1))=1
	endelse
endif
if keyword_set(bins) then bins2=bins

if ndimen(bins2) ne 2 then bins2=ebins2#bins2

data = dat.data*bins2
energy = dat.energy
denergy = dat.denergy
theta = dat.theta/!radeg
phi = dat.phi/!radeg
dtheta = dat.dtheta/!radeg
dphi = dat.dphi/!radeg
domega = dat.domega
	if ndimen(domega) eq 1 then domega=replicate(1.,dat.nenergy)#domega
mass = dat.mass
Const = 2./mass/mass*1e5
Const2 = 1.6e-12
charge=1.
value=0 & str_element,dat,'charge',value
if value ne 0 then charge=dat.charge		
if ((value eq 0) and (dat.mass lt 0.00010438871)) then charge=-1.		; this line works for Wind which does not have dat.charge
value=0 & str_element,dat,'sc_pot',value
if value ne 0 then energy=energy+(charge*dat.sc_pot/abs(charge))>0.		; energy/charge analyzer

solid_angle_corr=4.*!pi/total(domega(0,*))	; this should be correct in the structure
if (solid_angle_corr lt .99 or solid_angle_corr gt 1.01) and max(theta) gt 1.2 then print,'Error in dat.domega'   
solid_angle_corr=1.

eflux3dx = Const2*Const*total(denergy*(energy)^2*data*(dtheta/2.+cos(2*theta)*sin(dtheta)/2.)*(2.*sin(dphi/2.)*cos(phi)))
eflux3dy = Const2*Const*total(denergy*(energy)^2*data*(dtheta/2.+cos(2*theta)*sin(dtheta)/2.)*(2.*sin(dphi/2.)*sin(phi)))
eflux3dz = Const2*Const*total(denergy*(energy)^2*data*(2.*sin(theta)*cos(theta)*sin(dtheta/2.)*cos(dtheta/2.))*dphi)

return, [eflux3dx,eflux3dy,eflux3dz]

; units are ergs/cm^2-sec

end
;+
;FUNCTION:	n_3d_new(dat,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
;INPUT:	
;	dat:	structure,	2d data structure filled by get_eesa_surv, get_eesa_burst, etc.
;KEYWORDS
;	ENERGY:	fltarr(2),	optional, min,max energy range for integration
;	ERANGE:	fltarr(2),	optional, min,max energy bin numbers for integration
;	EBINS:	bytarr(na),	optional, energy bins array for integration
;					0,1=exclude,include,  
;					na = dat.nenergy
;	ANGLE:	fltarr(2,2),	optional, angle range for integration
;				theta min,max (0,0),(1,0) -90<theta<90 
;				phi   min,max (0,1),(1,1)   0<phi<360 
;	ARANGE:	fltarr(2),	optional, min,max angle bin numbers for integration
;	BINS:	bytarr(nb),	optional, angle bins array for integration
;					0,1=exclude,include,  
;					nb = dat.ntheta
;	BINS:	bytarr(na,nb),	optional, energy/angle bins array for integration
;					0,1=exclude,include
;PURPOSE:
;	Returns the density, n, 1/cm^3, corrects for spacecraft potential if dat.sc_pot exists
;NOTES:	
;	Function normally called by "get_3dt" or "get_2dt" to 
;	generate time series data for "tplot.pro".
;
;CREATED BY:
;	J.McFadden	00-2-24	
;LAST MODIFICATION:
;-
function n_3d_new,dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins

density = 0.

if dat2.valid eq 0 then begin
  print,'Invalid Data'
  return, density
endif

dat = conv_units(dat2,"df")		; Use distribution function
na = dat.nenergy
nb = dat.nbins

ebins2=replicate(1b,na)
if keyword_set(en) then begin
	ebins2(*)=0
	er2=[energy_to_ebin(dat,en)]
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(er) then begin
	ebins2(*)=0
	er2=er
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(ebins) then ebins2=ebins

bins2=replicate(1b,nb)
if keyword_set(an) then begin
	if ndimen(an) ne 2 then begin
		print,'Error - angle keyword must be (2,2)'
	endif else begin
		bins2=angle_to_bins(dat,an)
	endelse
endif
if keyword_set(ar) then begin
	bins2(*)=0
	if ar(0) gt ar(1) then begin
		bins2(ar(0):nb-1)=1
		bins2(0:ar(1))=1
	endif else begin
		bins2(ar(0):ar(1))=1
	endelse
endif
if keyword_set(bins) then bins2=bins

if ndimen(bins2) ne 2 then bins2=ebins2#bins2

data = dat.data*bins2
energy = dat.energy
denergy = dat.denergy
theta = dat.theta/!radeg
phi = dat.phi/!radeg
dtheta = dat.dtheta/!radeg
dphi = dat.dphi/!radeg
domega = dat.domega
	if ndimen(domega) eq 1 then domega=replicate(1.,dat.nenergy)#domega
mass = dat.mass 
Const = (mass)^(-1.5)*(2.)^(.5)
charge=1.
value=0 & str_element,dat,'charge',value
if value ne 0 then charge=dat.charge		
if ((value eq 0) and (dat.mass lt 0.00010438871)) then charge=-1.		; this line works for Wind which does not have dat.charge
value=0 & str_element,dat,'sc_pot',value
if value gt 0 or value lt 0 then energy=energy+(charge*dat.sc_pot/abs(charge))>0.		; energy/charge analyzer

solid_angle_corr=4.*!pi/total(domega(0,*))	; this should be correct in the structure
if (solid_angle_corr lt .99 or solid_angle_corr gt 1.01) and max(theta) gt 1.2 then print,'Error in dat.domega'   
solid_angle_corr=1.

;density = solid_angle_corr*Const*total(denergy*(energy^(.5))*data*domega)	
;density = Const*total(denergy*(energy^(.5))*data*domega)	
density = Const*total(denergy*(energy^(.5))*data*2.*cos(theta)*sin(dtheta/2.)*dphi)	

; units are 1/cm^3

return, density
end

;+
;FUNCTION:	p_3d_new(dat,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
;INPUT:	
;	dat:	structure,	2d data structure filled by get_eesa_surv, get_eesa_burst, etc.
;KEYWORDS
;	ENERGY:	fltarr(2),	optional, min,max energy range for integration
;	ERANGE:	fltarr(2),	optional, min,max energy bin numbers for integration
;	EBINS:	bytarr(na),	optional, energy bins array for integration
;					0,1=exclude,include,  
;					na = dat.nenergy
;	ANGLE:	fltarr(2,2),	optional, angle range for integration
;				theta min,max (0,0),(1,0) -90<theta<90 
;				phi   min,max (0,1),(1,1)   0<phi<360 
;	ARANGE:	fltarr(2),	optional, min,max angle bin numbers for integration
;	BINS:	bytarr(nb),	optional, angle bins array for integration
;					0,1=exclude,include,  
;					nb = dat.ntheta
;	BINS:	bytarr(na,nb),	optional, energy/angle bins array for integration
;					0,1=exclude,include
;PURPOSE:
;	Returns the pressure tensor, [Pxx,Pyy,Pzz,Pxy,Pxz,Pyz], eV/cm^3 
;NOTES:	
;	Function normally called by "get_3dt" or "get_2dt" to
;	generate time series data for "tplot.pro".
;
;CREATED BY:
;	J.McFadden	00-2-24	
;LAST MODIFICATION:
;	J.McFadden	05-2-8		Fixed diagonalization
;	J.McFadden	06-2-23		changed the s/c pot calculation to the same as n_2d_new.pro
;-
function p_3d_new,dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins

p3dxx = 0.
p3dyy = 0.
p3dzz = 0.
p3dxy = 0.
p3dxz = 0.
p3dyz = 0.

if dat2.valid eq 0 then begin
  print,'Invalid Data'
  return, [p3dxx,p3dyy,p3dzz,p3dxy,p3dxz,p3dyz]
endif

dat = conv_units(dat2,"df")		; Use Energy Flux
na = dat.nenergy
nb = dat.nbins
	if dat.data_name eq 'Pesa High' and dat.nbins eq 97 then dat.data(*,96)=0.
	
ebins2=replicate(1b,na)
if keyword_set(en) then begin
	ebins2(*)=0
	er2=[energy_to_ebin(dat,en)]
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(er) then begin
	ebins2(*)=0
	er2=er
	if er2(0) gt er2(1) then er2=reverse(er2)
	ebins2(er2(0):er2(1))=1
endif
if keyword_set(ebins) then ebins2=ebins

bins2=replicate(1b,nb)
if keyword_set(an) then begin
	if ndimen(an) ne 2 then begin
		print,'Error - angle keyword must be (2,2)'
	endif else begin
		bins2=angle_to_bins(dat,an)
	endelse
endif
if keyword_set(ar) then begin
	bins2(*)=0
	if ar(0) gt ar(1) then begin
		bins2(ar(0):nb-1)=1
		bins2(0:ar(1))=1
	endif else begin
		bins2(ar(0):ar(1))=1
	endelse
endif
if keyword_set(bins) then bins2=bins

if ndimen(bins2) ne 2 then bins2=ebins2#bins2

data = dat.data*bins2
energy = dat.energy
denergy = dat.denergy
theta = dat.theta/!radeg
phi = dat.phi/!radeg
dtheta = dat.dtheta/!radeg
dphi = dat.dphi/!radeg
domega = dat.domega
	if ndimen(domega) eq 1 then domega=replicate(1.,dat.nenergy)#domega
;mass = dat.mass * 1.6e-22
mass = dat.mass 
;Const = (mass/(2.*1.6e-12))^(-.5)

charge=1.
value=0 & str_element,dat,'charge',value
if value ne 0 then charge=dat.charge		
if ((value eq 0) and (dat.mass lt 0.00010438871)) then charge=-1.		; this line works for Wind which does not have dat.charge
value=0 & str_element,dat,'sc_pot',value
if value ne 0 then energy=energy+(charge*dat.sc_pot/abs(charge))>0.		; energy/charge analyzer

solid_angle_corr=4.*!pi/total(domega(0,*))	; this should be correct in the structure
if (solid_angle_corr lt .99 or solid_angle_corr gt 1.01) and max(theta) gt 1.2 then print,'Error in dat.domega'   
solid_angle_corr=1.

th1=theta-dtheta/2.
th2=theta+dtheta/2.
ph1=phi-dphi/2.
ph2=phi+dphi/2.
cth1 = cos(th1)
cth2 = cos(th2)
sth1 = sin(th1)
sth2 = sin(th2)
cph1 = cos(ph1)
cph2 = cos(ph2)
sph1 = sin(ph1)
sph2 = sin(ph2)
s_2ph1 = sin(2.*ph1)
s_2ph2 = sin(2.*ph2)
s2_ph1 = sph1^2
s2_ph2 = sph2^2
s3_th1 = sth1^3
s3_th2 = sth2^3 
c3_th1 = cth1^3
c3_th2 = cth2^3 

Const = (1.d*mass)^(-2.5)*(2.)^1.5
p3dxx = Const*total(denergy*(energy^(1.5))*data*((ph2-ph1)/2.+(s_2ph2-s_2ph1)/4.)*(sth2-sth1-(s3_th2-s3_th1)/3.))
p3dyy = Const*total(denergy*(energy^(1.5))*data*((ph2-ph1)/2.-(s_2ph2-s_2ph1)/4.)*(sth2-sth1-(s3_th2-s3_th1)/3.))
p3dzz = Const*total(denergy*(energy^(1.5))*data*dphi*(s3_th2-s3_th1)/3.)
p3dxy = Const*total(denergy*(energy^(1.5))*data*((s2_ph2-s2_ph1)/2.)*(sth2-sth1-s3_th2+s3_th1))
p3dxz = Const*total(denergy*(energy^(1.5))*data*(sph2-sph1)*((c3_th1-c3_th2)/3.))
p3dyz = Const*total(denergy*(energy^(1.5))*data*(cph1-cph2)*((c3_th1-c3_th2)/3.))
;print,p3dxx,p3dyy,p3dzz,p3dxy,p3dxz,p3dyz
;print,total(((ph2-ph1)/2.+(s_2ph2-s_2ph1)/4.)*(sth2-sth1-(s3_th2-s3_th1)/3.))
;print,total(((ph2-ph1)/2.-(s_2ph2-s_2ph1)/4.)*(sth2-sth1-(s3_th2-s3_th1)/3.))
;print,total(dphi*(s3_th2-s3_th1)/3.)
;print,total(((s2_ph2-s2_ph1)/2.)*(sth2-sth1-s3_th2+s3_th1))
;print,total((sph2-sph1)*((c3_th1-c3_th2)/3.))
;print,total((cph1-cph2)*((c3_th1-c3_th2)/3.))

flux=j_3d_new(dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
density=n_3d_new(dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
vel = flux/density

p3dxx = mass*(p3dxx-vel(0)*flux(0)/1.e10)
p3dyy = mass*(p3dyy-vel(1)*flux(1)/1.e10)
p3dzz = mass*(p3dzz-vel(2)*flux(2)/1.e10)
p3dxy = mass*(p3dxy-vel(0)*flux(1)/1.e10)
p3dxz = mass*(p3dxz-vel(0)*flux(2)/1.e10)
p3dyz = mass*(p3dyz-vel(1)*flux(2)/1.e10)

; Rotate the tensor about the magnetic field to diagonalize
; This should give a result that diagonalizes the pressure tensor about dat2.magf
; where magf is in s/c coordinates -- ie same as the dat2.theta and dat2.phi coordinates.

; First form the pressure tensor
	p = [[p3dxx,p3dxy,p3dxz],[p3dxy,p3dyy,p3dyz],[p3dxz,p3dyz,p3dzz]]
;	print,p
; Rotate p about Z-axis by the angle between X and the projection of B on the XY plane
;	print,dat2.magf
if total(dat2.magf*dat2.magf) ne 0. then begin
	bx=dat2.magf(0)
	by=dat2.magf(1)
	bz=dat2.magf(2)
	ph=atan(by,bx)
;	print,ph
	rot_ph=([[cos(ph),-sin(ph),0],[sin(ph),cos(ph),0],[0,0,1]])
	p = rot_ph#p#transpose(rot_ph)
;	print,p
; Then rotate p about Y-axis by the angle between Bz and B 
	th=!pi/2.-atan(bz,(bx^2+by^2)^.5)
;	print,th
	rot_th=[[cos(th),0,sin(th)],[0,1,0],[-sin(th),0,cos(th)]]
	p = rot_th#p#transpose(rot_th)
endif 

;	Pressure is in units of eV/cm**3

;return, [p3dxx,p3dyy,p3dzz,p3dxy,p3dxz,p3dyz]
return, [p(0,0),p(1,1),p(2,2),p(0,1),p(0,2),p(1,2)]

end

;+
;FUNCTION:	t_3d_new(dat,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
;INPUT:	
;	dat:	structure,	2d data structure filled by get_eesa_surv, get_eesa_burst, etc.
;KEYWORDS
;	ENERGY:	fltarr(2),	optional, min,max energy range for integration
;	ERANGE:	fltarr(2),	optional, min,max energy bin numbers for integration
;	EBINS:	bytarr(na),	optional, energy bins array for integration
;					0,1=exclude,include,  
;					na = dat.nenergy
;	ANGLE:	fltarr(2,2),	optional, angle range for integration
;				theta min,max (0,0),(1,0) -90<theta<90 
;				phi   min,max (0,1),(1,1)   0<phi<360 
;	ARANGE:	fltarr(2),	optional, min,max angle bin numbers for integration
;	BINS:	bytarr(nb),	optional, angle bins array for integration
;					0,1=exclude,include,  
;					nb = dat.ntheta
;	BINS:	bytarr(na,nb),	optional, energy/angle bins array for integration
;					0,1=exclude,include
;PURPOSE:
;	Returns the temperature, [Tx,Ty,Tz,Tavg], eV 
;NOTES:	
;	Function normally called by "get_3dt" or "get_2dt" to
;	generate time series data for "tplot.pro".
;
;CREATED BY:
;	J.McFadden	00-2-24
;LAST MODIFICATION:
;-
function t_3d_new,dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins

Tavg = 0.
Tx = 0.
Ty = 0.
Tz = 0.

if dat2.valid eq 0 then begin
	print,'Invalid Data'
	return, [Tx,Ty,Tz,Tavg]
endif

press = p_3d_new(dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
density = n_3d_new(dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
if density ne 0. then begin
	Tavg = (press(0)+press(1)+press(2))/(density*3.)
	Tx = press(0)/(density)
	Ty = press(1)/(density)
	Tz = press(2)/(density)
endif

return, [Tx,Ty,Tz,Tavg]

end

;+
;FUNCTION:	v_3d_new(dat,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
;INPUT:	
;	dat:	structure,	2d data structure filled by get_eesa_surv, get_eesa_burst, etc.
;KEYWORDS
;	ENERGY:	fltarr(2),	optional, min,max energy range for integration
;	ERANGE:	fltarr(2),	optional, min,max energy bin numbers for integration
;	EBINS:	bytarr(na),	optional, energy bins array for integration
;					0,1=exclude,include,  
;					na = dat.nenergy
;	ANGLE:	fltarr(2,2),	optional, angle range for integration
;				theta min,max (0,0),(1,0) -90<theta<90 
;				phi   min,max (0,1),(1,1)   0<phi<360 
;	ARANGE:	fltarr(2),	optional, min,max angle bin numbers for integration
;	BINS:	bytarr(nb),	optional, angle bins array for integration
;					0,1=exclude,include,  
;					nb = dat.ntheta
;	BINS:	bytarr(na,nb),	optional, energy/angle bins array for integration
;					0,1=exclude,include
;PURPOSE:
;	Returns the velocity, [Vx,Vy,Vz], km/s 
;NOTES:	
;	Function normally called by "get_3dt" or "get_2dt" to
;	generate time series data for "tplot.pro".
;
;CREATED BY:
;	J.McFadden	02-8-28	
;LAST MODIFICATION:
;-
function v_3d_new,dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins

vel = [0.,0.,0.]

if dat2.valid eq 0 then begin
	print,'Invalid Data'
	return, vel
endif

flux = j_3d_new(dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
density = n_3d_new(dat2,ENERGY=en,ERANGE=er,EBINS=ebins,ANGLE=an,ARANGE=ar,BINS=bins)
if density ne 0. then vel = 1.e-5*flux/density

return, vel

end
;+
; Purpose: To make mission overview plots of all instruments
;
; Example: thm_ui_temp_overplot,sc='a',trange =
; ['2007-06-04','2007-06-05']
;$LastChangedBy$
;$LastChangedDate$
;$LastChangedRevision$
;$URL$
;-

pro thm_ui_temp_overplot, sc = sc0, trange = trange, varnames_out = tvn, $
                          progobj = progobj

  If(obj_valid(progobj)) Then update = 1b Else update = 0b

  sc = strcompress(strlowcase(sc0), /remove_all)
  tt = time_double(trange)
  t1xxx = str2time(strmid(time_string(tt[0]), 0, 10))
  t2xxx = str2time(strmid(time_string(tt[1]-1.), 0, 10))
  ndays = 1+fix((t2xxx-t1xxx)/(24.*3600.))
  timespan, t1xxx, ndays
  date = time_string(t1xxx)

;  del_data, '*'                 ; give ourselves a clean slate

;load all data
;-----------------------------
  If(update) Then progobj -> update, 0.0, text = 'Loading State Data'

  thm_load_state, probe = sc, /get_support
  If(update) Then progobj -> update, 0.0, text = 'Loading Fit Data'
  thm_load_fit, probe = sc, /get_support
  thm_cal_fit

  If(update) Then progobj -> update, 0.0, text = 'Loading FBK Data'
  thm_load_fbk, /get_supp, probe = sc
  thm_cal_fbk, /verbose, probe = sc

  If(update) Then progobj -> update, 0.0, text = 'Loading SST Data'
  thm_load_sst2, probe = sc
  thm_load_hsk, probe = sc, varformat = 'th?_*sst* th?_imon_?5va th?_imon_?5va'
  thm_load_state2, probe = sc, /get_support, /polar

  If(update) Then progobj -> update, 0.0, text = 'Loading ESA Data'
  thm_load_esa_pkt, probe = sc

  If(update) Then progobj -> update, 0.0, text = 'Loading GMAG Data'
  gmag_sites = ["gbay", "fsmi", "fykn"]
  gmag_sites_Qday_Xmedian = [16416., 10381., 10728.]
  thm_load_gmag, site = gmag_sites

  If(update) Then progobj -> update, 0.0, text = 'Loading ASK Data'
  thm_load_ask, /verbose
  asi_sites = tnames('*ask*')
	
  If(update) Then progobj -> update, 0.0, text = 'Loading HSK Data'
  thm_load_hsk, probe = sc, varformat = 'th'+sc+'*issr_mode*'
    
  inames = tnames('th'+sc+'_*')
  data_ss = sswhere_arr(tnames(), inames)
  If(data_ss[0] Ne -1) Then thm_ui_only_trange, tt[0], tt[1], data_ss
  inames = tnames('thg_*')
  data_ss = sswhere_arr(tnames(), inames)
  If(data_ss[0] Ne -1) Then thm_ui_only_trange, tt[0], tt[1], data_ss

;kluge to prevent missing data from crashing things
  index_fit = where(strjoin('th'+sc+'_fit_b') eq tnames())
  index_state = where(strjoin('th'+sc+'_state_spinras'))
  if (index_fit(0) eq -1 or index_state(0) eq -1) then begin
    store_data, strjoin('th'+sc+'_fit_b_gse'), data = {x:[time_double(date)], y:[[float('NaN')], [float('NaN')], [float('NaN')]]}
  endif else begin
    thm_cotrans, strjoin('th'+sc+'_fit_b'), out_suf = '_gse', in_c = 'dsl', out_c = 'gse'
  endelse

;clip data
  get_data, strjoin('th'+sc+'_fit_b_gse'), data = d
  index = where(d.y gt 100 or d.y lt -100)
  if (index(0) ne -1) then d.y(index) = float('NaN')
  store_data, strjoin('th'+sc+'_fit_b_gse'), data = d

  name = 'th'+sc+'_fit_b_gse'
  options, name, 'ytitle', 'B FIT!CGSE (nT)'
  options, name, 'labels', ['Bx', 'By', 'Bz']
  options, name, 'labflag', 1
  options, name, 'colors', [2, 4, 6]


;load SCM data
;--------------

;kluge to prevent missing data from crashing things 
  get_data, strjoin('th'+sc+'_fb1_raw'), data = dd
  if size(dd, /type) ne 8 then begin
    filler = fltarr(1, 6)
    filler(0, *) = float('NaN')
    store_data, strjoin('th'+sc+'_fb1_raw'), data = {x:[time_double(date)], y:filler, v:fltarr(6)}
    store_data, strjoin('th'+sc+'_fb2_raw'), data = {x:[time_double(date)], y:filler, v:fltarr(6)}
  endif 

  options, 'th'+sc+'_fb1_raw', 'spec', 1
  options, 'th'+sc+'_fb2_raw', 'spec', 1
  options, 'th'+sc+'_fb1_raw', 'ytitle', 'tha_fb1_raw'
  options, 'th'+sc+'_fb2_raw', 'ytitle', 'tha_fb2_raw'



;load SST spectrograms 
;----------------------
  If(update) Then progobj -> update, 0.0, text = 'Calculating SST particle spectra'
  mtypes = ['sif', 'sef']
  thm_part_spec_calc, probe = sc, comps = ['density', 'flux', 'mftens', 'velocity', 'T3'], types = mtypes

;kluge to prevent missing data from crashing things 
  index_sst = where(strjoin('th'+sc+'_psif_spec') eq tnames())
  if index_sst eq -1 then begin
    filler = fltarr(1, 16)
    filler(0, *) = float('NaN')
    store_data, strjoin('th'+sc+'_psif_spec'), data = {x:[time_double(date)], y:filler, v:fltarr(16)}
    store_data, strjoin('th'+sc+'_psef_spec'), data = {x:[time_double(date)], y:filler, v:fltarr(16)}
  endif

;SST ion panel
  name = 'th'+sc+'_psif_spec'
  options, name, 'spec', 1
  options, name, 'ytitle', 'th'+sc+'!CSST ions!Cspec'
  options, name, 'ztitle', 'Eflux !C eV/cm!U2!N!C-s-sr-eV'
  options, name, 'y_no_interp', 1
  options, name, 'x_no_interp', 1
;SST electron panel
  name = 'th'+sc+'_psef_spec'
  options, name, 'spec', 1
  options, name, 'ytitle', 'th'+sc+'!CSST elec!Cspec'
  options, name, 'ztitle', 'Eflux !C eV/cm!U2!N!C-s-sr-eV'
  options, name, 'y_no_interp', 1
  options, name, 'x_no_interp', 1


;load ESA spectrograms and moments
;----------------------------------

; choose type of data - f-full, r-reduced, b-burst
  typ = 'r'
  if typ eq 'f' then gap_time = 1000. else gap_time = 10.

; decide if you want magnetosheath (sheath=1) or plasmasheet (sheath=0) limits
  sheath = 0
  if sheath then emin = 10. else emin = 30.
  if sheath then nmax = 100. else nmax = 10.

; Get position info in gse coordinates

  cotrans, strjoin('th'+sc+'_state_pos'), strjoin('th'+sc+'_pos_gse'), /GEI2GSE
  
  get_data, strjoin('th'+sc+'_pos_gse'), data = tmp
  store_data, strjoin('th'+sc+'_pos_gse_x'), data = {x:tmp.x, y:tmp.y(*, 0)/6370.}
  options, strjoin('th'+sc+'_pos_gse_x'), 'ytitle', 'Xgse'
  store_data, strjoin('th'+sc+'_pos_gse_y'), data = {x:tmp.x, y:tmp.y(*, 1)/6370.}
  options, strjoin('th'+sc+'_pos_gse_y'), 'ytitle', 'Ygse'
  store_data, strjoin('th'+sc+'_pos_gse_z'), data = {x:tmp.x, y:tmp.y(*, 2)/6370.}
  options, strjoin('th'+sc+'_pos_gse_z'), 'ytitle', 'Zgse'

; ion plots
;kluge to prevent missing data from crashing things
  index_esai = where(strjoin('th'+sc+'_pei'+typ+'_en_counts') eq tnames())
  if index_esai(0) eq -1 then begin 
    filler = fltarr(1, 31)
    filler(0, *) = float('Nan')
    store_data, strjoin('th'+sc+'_pei'+typ), data = {x:[time_double(date)], y:fltarr(1, 31), v:fltarr(31)} 
    store_data, strjoin('Ni_th'+sc+'_pei'+typ), data = {x:[time_double(date)], y:[float('Nan')]}
    store_data, strjoin('Vi_th'+sc+'_pei'+typ), data = {x:[time_double(date)], y:[[float('NaN')], [float('NaN')], [float('NaN')]]}
    filler = fltarr(1, 6)
    filler(0, *) = float('Nan')
    store_data, strjoin('Ti_th'+sc+'_pei'+typ), data = {x:[time_double(date)], y:fltarr(1, 6)}
  endif 

  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA energy spectra'
	
  get_dat = strjoin('th'+sc+'_pei'+typ)
  name1 = strjoin('th'+sc+'_pei'+typ)
  if index_esai(0) ne -1 then get_en_spec, get_dat, units = 'eflux', retrace = 1, name = name1, gap_time = gap_time, t1 = t1, t2 = t2
  zlim, name1, 1.e3, 1.e7, 1
  ylim, name1, 3., 40000., 1
  options, name1, 'ztitle', 'Eflux !C eV/cm!U2!N!C-s-sr-eV'
  options, name1, 'ytitle', 'th'+sc+'!CESA ions!CeV'
  options, name1, 'spec', 1
  options, name1, 'x_no_interp', 1
  options, name1, 'y_no_interp', 1

  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA ion density'
  name1 = strjoin('Ni_th'+sc+'_pei'+typ)
  if index_esai(0) ne -1 then get_2dt, 'n_3d_new', get_dat, name = name1, gap_time = gap_time, t1 = t1, t2 = t2, energy = [20., 21000.]
  ylim, name1, .1, nmax, 1
  options, name1, 'ytitle', 'Ni th'+sc+'!C1/cm!U3'
                                ;options,name1,labels=['N!di!n']
	
  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA ion velosity'
  name1 = strjoin('Vi_th'+sc+'_pei'+typ)
  if index_esai(0) ne -1 then get_2dt, 'v_3d_new', get_dat, name = name1, gap_time = gap_time, t1 = t1, t2 = t2, energy = [20., 21000.]
  options, name1, 'ytitle', 'Vi th'+sc+'!Ckm/s'
  options, name1, 'colors', [2, 4, 6] ;[cols.blue,cols.green,cols.red]
  options, name1, labels = ['Vi!dx!n', 'Vi!dy!n', 'Vi!dz!n'], constant = 0.
  options, name1, 'labflag', 1

  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA ion Temp'
  name1 = strjoin('Ti_th'+sc+'_pei'+typ)
  if index_esai(0) ne -1 then get_2dt, 't_3d_new', get_dat, name = name1, gap_time = gap_time, t1 = t1, t2 = t2, energy = [20., 21000.]
  ylim, name1, 100, 10000., 1
  if sheath then ylim, name1, 10, 10000., 1
  options, name1, 'ytitle', 'Ti th'+sc+'!CeV'
	
; electron plots

;kluge to prevent missing data from crashing things
  index_esae = where(strjoin('th'+sc+'_pee'+typ+'_en_counts') eq tnames())
  if index_esae(0) eq -1 then begin 
    filler = fltarr(1, 31)
    filler(0, *) = float('Nan')
    store_data, strjoin('th'+sc+'_pee'+typ), data = {x:[time_double(date)], y:filler, v:fltarr(31)} 
    store_data, strjoin('Ne_th'+sc+'_pee'+typ), data = {x:[time_double(date)], y:[float('Nan')]}
    store_data, strjoin('Ve_th'+sc+'_pee'+typ), data = {x:[time_double(date)], y:[[float('NaN')], [float('NaN')], [float('NaN')]]}
    filler = fltarr(1, 6)
    filler(0, *) = float('Nan')
    store_data, strjoin('Te_th'+sc+'_pee'+typ), data = {x:[time_double(date)], y:filler}
  endif 

  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA electron spectra'
  get_dat = strjoin('th'+sc+'_pee'+typ)
  name1 = strjoin('th'+sc+'_pee'+typ)
  if index_esae(0) ne -1 then get_en_spec, get_dat, units = 'eflux', retrace = 1, name = name1, gap_time = gap_time, t1 = t1, t2 = t2
  zlim, name1, 1.e5, 1.e9, 1
  ylim, name1, 3., 40000., 1
  options, name1, 'ztitle', 'Eflux !C eV/cm!U2!N!C-s-sr-eV'
  options, name1, 'ytitle', 'th'+sc+'!CESA elec!CeV'
  options, name1, 'spec', 1
  options, name1, 'x_no_interp', 1
  options, name1, 'y_no_interp', 1

  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA electron density'
  name1 = strjoin('Ne_th'+sc+'_pee'+typ)
  if index_esae(0) ne -1 then get_2dt, 'n_3d_new', get_dat, name = name1, gap_time = gap_time, t1 = t1, t2 = t2, energy = [emin, 27000.]
  ylim, name1, .1, nmax, 1
  options, name1, 'ytitle', 'Ne th'+sc+'!C1/cm!U3'
	;options,name1,labels=['N!de!n']
	
  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA electron velocity'
  name1 = strjoin('Ve_th'+sc+'_pee'+typ)
  if index_esae(0) ne -1 then get_2dt, 'v_3d_new', get_dat, name = name1, gap_time = gap_time, t1 = t1, t2 = t2, energy = [emin, 27000.]
  options, name1, 'ytitle', 'Ve th'+sc+'!Ckm/s'
  options, name1, 'colors', [2, 4, 6] ;[cols.blue,cols.green,cols.red]
  options, name1, labels = ['V!dex!n', 'V!dey!n', 'V!dez!n'], constant = 0.
  options, name1, 'labflag', 1

  If(update) Then progobj -> update, 0.0, text = 'Calculating ESA electron T'
  name1 = strjoin('Te_th'+sc+'_pee'+typ)
  if index_esae(0) ne -1 then get_2dt, 't_3d_new', get_dat, name = name1, gap_time = gap_time, t1 = t1, t2 = t2, energy = [emin, 27000.]
  ylim, name1, 100, 10000., 1
  if sheath then ylim, name1, 10, 10000., 1
  options, name1, 'ytitle', 'Te th'+sc+'!CeV'
	

; plot quantities (manipulating the plot quantities for the sake of plot aesthetics)
  If(update) Then progobj -> update, 0.0, text = 'Setting up plots'
;kluge for labeling the density
  get_data, strjoin('Ne_th'+sc+'_pee'+typ), data = d
  dummy = fltarr(n_elements(d.y), 2)
  dummy(*, 0) = d.y
  dummy(*, 1) = d.y
  Ne_kluge_name = strjoin('Ne_th'+sc+'_pee'+typ+'_kluge')
  store_data, Ne_kluge_name, data = {x:d.x, y:dummy}
  options, Ne_kluge_name, labels = ['Ni', 'Ne']
  options, Ne_kluge_name, colors = [0, 6]
  options, Ne_kluge_name, 'labflag', 1

  store_data, strjoin('th'+sc+'_Nie'), data = [strjoin('Ni_th'+sc+'_pei'+typ), Ne_kluge_name]
  options, strjoin('th'+sc+'_Nie'), 'ytitle', 'Ni,e th'+sc+'!C1/cm!U3'
    
  store_data, strjoin('th'+sc+'_Tie'), data = [strjoin('Ti_th'+sc+'_pei'+typ), strjoin('Te_th'+sc+'_pee'+typ)]
  options, strjoin('th'+sc+'_Tie'), 'ytitle', 'Ti,e th'+sc



; load gmag data
;----------------

  site_tvars = tnames('thg_mag*')
    
  if site_tvars(0) ne '' then begin
    
      ;a hideous kluge to get tplot to format the way I want it to format (basically stick colored labels evenly spaced on the right)
    store_data, 'thg_mag_kluge', data = {x:[time_double(date)], y:[[float('NaN')], [float('NaN')], [float('NaN')]]}
    options, 'thg_mag_kluge', 'colors', [0, 2, 4]  
    options, 'thg_mag_kluge', 'labels', gmag_sites
    options, 'thg_mag_kluge', 'labflag', 1  
      
    for i = 0, n_elements(site_tvars)-1 do begin

      site_name = strmid(site_tvars(i), 8, 4)
      index_site = where(gmag_sites eq site_name)
      get_data, site_tvars(i), thg_mag_t, thg_mag_d
      gmag_values_d = thg_mag_d(*, 0)-gmag_sites_Qday_Xmedian(index_site(0))
      index = where(gmag_values_d gt 1000. or gmag_values_d lt -1000.)
      if (index(0) ne -1) then gmag_values_d(index) = float('NaN')
      store_data, site_tvars(i)+'_d', data = {x:thg_mag_t, y:gmag_values_d}
      options, site_tvars(i)+'_d', 'colors', index_site(0)*2  
                          
    endfor
      
    store_data, 'THEMIS_AU/AL_index', data = ['thg_mag_kluge', tnames('thg_mag*_d')]
        
  endif else begin
    store_data, 'THEMIS_AU/AL_index', data = {x:[time_double(date)], y:[[float('NaN')], [float('NaN')], [float('NaN')]]}
    ylim, 'THEMIS_AU/AL_index', -500., 500, 0
  endelse
    
  options, 'THEMIS_AU/AL_index', 'ytitle', 'THEMIS!CAU/AL index!CnT'


; load ASK data and plot 3 specific ones (can be changed)
;---------------------------------------------------------

  if asi_sites(0) ne '' then copy_data, asi_sites(0), 'Keogram' else store_data, 'Keogram', data = {x:[time_double(date)], y:float('NaN')}


; make tplot variable tracking the sample rate (0=SS,1=FS,2=PB,3=WB)
;-------------------------------------------------------------------

  if tnames('th'+sc+'*issr_mode*') eq '' then store_data, strjoin('th'+sc+'_hsk_issr_mode_raw'), data = {x:[time_double(date)], y:float('Nan')}
  get_data, strjoin('th'+sc+'_hsk_issr_mode_raw'), data = d
  ss_data = float(d.y)
  index_ss_fill = where(d.y ne 0)
  index_ss = where(d.y eq 0)
  if (index_ss_fill(0) ne -1) then ss_data(index_ss_fill) = float('NaN')
  if (index_ss(0) ne -1) then ss_data(index_ss) = 0.

  fs_data = float(d.y)
  index_fs_fill = where(d.y ne 0 and d.y ne 1)
  index_fs = where(d.y eq 0 or d.y eq 1)
  if (index_fs_fill(0) ne -1) then fs_data(index_fs_fill) = float('NaN')
  if (index_fs(0) ne -1) then fs_data(index_fs) = 0.

  pb_data = float(d.y)
  index_pb_fill = where(d.y ne 0 and d.y ne 1 and d.y ne 2)
  index_pb = where(d.y eq 0 or d.y eq 1 or d.y eq 2)
  if (index_pb_fill(0) ne -1) then pb_data(index_pb_fill) = float('NaN')
  if (index_pb(0) ne -1) then pb_data(index_pb) = 0.

  pb_data2 = float(d.y)         ; pb_data2 is for symbols below bar
  index_pb_fill2 = where(d.y ne 2)
  index_pb2 = where(d.y eq 2)
  if (index_pb_fill2(0) ne -1) then pb_data2(index_pb_fill2) = float('NaN')
  if (index_pb2(0) ne -1) then pb_data2(index_pb2) = -1.

  wb_data = float(d.y)
  index_wb_fill = where(d.y ne 0 and d.y ne 1 and d.y ne 2 and d.y ne 3)
  index_wb = where(d.y eq 0 or d.y eq 1 or d.y eq 2 or d.y eq 3)
  if (index_wb_fill(0) ne -1) then wb_data(index_wb_fill) = float('NaN')
  if (index_wb(0) ne -1) then wb_data(index_wb) = 0.

  wb_data2 = float(d.y)         ; wb_data2 is for symbols above bar
  index_wb_fill2 = where(d.y ne 3)
  index_wb2 = where(d.y eq 3)
  if (index_wb_fill2(0) ne -1) then wb_data2(index_wb_fill2) = float('NaN')
  if (index_wb2(0) ne -1) then wb_data2(index_wb2) = 1.  


  store_data, 'slow_survey_bar', data = {x:d.x, y:ss_data}
  store_data, 'fast_survey_bar', data = {x:d.x, y:fs_data}
  store_data, 'particle_burst_bar', data = {x:d.x, y:pb_data}
  store_data, 'particle_burst_sym', data = {x:d.x, y:pb_data2}
  store_data, 'wave_burst_bar', data = {x:d.x, y:wb_data}
  store_data, 'wave_burst_sym', data = {x:d.x, y:wb_data2}
  store_data, 'aesthetic_bar', data = {x:d.x, y:ss_data}

  options, 'aesthetic_bar', 'color', 255
  options, 'slow_survey_bar', 'color', 5
  options, 'fast_survey_bar', 'color', 6
  options, 'particle_burst_bar', 'color', 3
  options, 'particle_burst_sym', 'color', 0
  options, 'wave_burst_bar', 'color', 0
  options, 'wave_burst_sym', 'color', 0

  options, 'slow_survey_bar', 'thick', 5
  options, 'fast_survey_bar', 'thick', 5
  options, 'particle_burst_bar', 'thick', 5
  options, 'particle_burst_sym', 'psym', 6
  options, 'particle_burst_sym', 'symsize', 0.2
  options, 'wave_burst_bar', 'thick', 5
  options, 'wave_burst_sym', 'psym', 6
  options, 'wave_burst_sym', 'symsize', 0.2

  store_data, 'sample_rate', data = ['aesthetic_bar', 'wave_burst_bar', 'particle_burst_bar', 'fast_survey_bar', 'slow_survey_bar', 'particle_burst_sym', 'wave_burst_sym']
  ylim, 'sample_rate', -1.1, 1.1, 0
  options, 'sample_rate', 'panel_size', 0.2




; final tplot preparations
;--------------------------

;calculate ESA and SST min/max values on various intervals
  esai_name = strjoin('th'+sc+'_pei'+typ)
  esae_name = strjoin('th'+sc+'_pee'+typ)
  ssti_name = strjoin('th'+sc+'_psif_spec')
  sste_name = strjoin('th'+sc+'_psef_spec')
  
  esasst_types = [esai_name, esae_name, ssti_name, sste_name]

  particle_minmax_z = dblarr(4, 5, 2) ; ([esai,esae,ssti,sste],[t0,t1,t2,t3,t4],[min,max])

  for j = 0, 3 do begin
    
    get_data, esasst_types(j), data = d

    for i = 0, 4 do begin
  
      starttime = time_double(date)+i*3600.*6.
      endtime = starttime+3600.*6.
    
      if i eq 4 then begin
        starttime = time_double(date)
        endtime = starttime+3600.*24.
      endif
    
      index1 = where(d.x ge starttime and d.x le endtime)
      if index1(0) ne -1 then begin
        data_clip = d.y(index1, *) 
        index2 = where(data_clip gt 1)
      endif else index2 = -1
  
      if index2(0) ne -1 then begin
        d_min = min(data_clip(index2)) 
        d_max = max(data_clip(index2))
      endif else begin 
        d_min = 1. 
        d_max = 100000. 
      endelse
    
      d_min = 10.^round(alog10(d_min))
      d_max = 10.^round(alog10(d_max))
  
      particle_minmax_z(j, i, 0) = d_min
      particle_minmax_z(j, i, 1) = d_max
    

    endfor                      ;i
    
  endfor                        ;j

;make ESA and SST ions on same scale

  for i = 0, 4 do begin
    particle_minmax_z(0, i, 0) = min([particle_minmax_z(0, i, 0), particle_minmax_z(2, i, 0)])
    particle_minmax_z(0, i, 1) = max([particle_minmax_z(0, i, 1), particle_minmax_z(2, i, 1)])
    particle_minmax_z(2, i, 0) = particle_minmax_z(0, i, 0)
    particle_minmax_z(2, i, 1) = particle_minmax_z(0, i, 1)
  endfor


; plot it!

;zlim,esai_name,particle_minmax_z(0,4,0),particle_minmax_z(0,4,1),1
;zlim,esae_name,particle_minmax_z(1,4,0),particle_minmax_z(1,4,1),1
;zlim,ssti_name,particle_minmax_z(2,4,0),particle_minmax_z(2,4,1),1
;zlim,sste_name,particle_minmax_z(3,4,0),particle_minmax_z(3,4,1),1

  zlim, esai_name, 1e3, 1e7, 1
  zlim, esae_name, 1e5, 1e9, 1
  zlim, ssti_name, 1e0, 1e5, 1
  zlim, sste_name, 1e0, 1e5, 1


  If(update) Then progobj -> update,   0.0,  text = 'Plotting..'
  tplot_options, 'lazy_ytitle', 0 ; prevent auto formatting on ytitle (namely having carriage returns at underscores)

  set_plot, 'x'
  window, 0, xsize = 850., ysize = 700, title = 'THEMIS Science Software Overview'
;set_plot,'z'   ; set_plot now done in thm_over_shell.pro
;device, set_resolution = [850, 900]

  !p.background = 255.
  !p.color = 0.
  time_stamp, /off
  loadct2, 43
  !p.charsize = 0.8

  tvn = ['THEMIS_AU/AL_index', 'Keogram', $
         'th'+sc+'_fit_b_gse', 'th'+sc+'_Nie', 'Vi_th'+sc+'_pei'+typ, 'th'+sc+'_Tie', 'sample_rate', $
         'th'+sc+'_psif_spec', 'th'+sc+'_pei'+typ, 'th'+sc+'_psef_spec', 'th'+sc+'_pee'+typ, $
         'th'+sc+'_fb1_raw', 'th'+sc+'_fb2_raw']

  tplot, trange = tt, tvn, title = 'THEMIS '+strupcase(sc), $
    var_label = ['th'+sc+'_pos_gse_z', 'th'+sc+'_pos_gse_y', 'th'+sc+'_pos_gse_x']

  If(update) Then progobj -> update,   0.0,  text = 'Finished overview for Probe:'+sc

end







