;+
;
;Procedure:	thm_l2gen_fbk
;
;Purpose:	Creates Level 2 data CDF file of THEMIS FBK data.
;		File includes EFI (Electric Field Instrument) and SCM (Search Coil Magnetometer) 
;			Spectra Data.
;		File includes all metadata (global and variable attributes) necessary 
;			to be ISTP compliant and QSAS compatible.
;		Program is intended primarily for generation of 1 day files.
;
;Inputs:	probe:	  A string indicating probe. 
;			  Valid names are : 'a', 'b', 'c', 'd' and 'e'.
;		date:  	  A string indicating the start time.
;			  Format: YYYY-MM-DD/hh:mm:ss
;		dur:	  Number indicating the duration.
;		days:	  A keyword that sets the units for dur to be day (default).
;		hours:	  A keyword that sets the units for dur to be hours.
;		minutes:  A keyword that sets the units for dur to be minutes.
;		noload:	  A keyword that prevents data from being loaded.
;			  Note: Data must already be loaded.
;		mastercdf:A string indicating the location of the mastercdf to be use.
;			  This master contains all the L2 metadata.
;
;Outputs:	File is created in local directory.
; 		Filename format:  thx_l2_fbk_YYYYMMDD_v01.cdf
;
;Example:	thm_l2gen_fbk,probe='a',date='2007-06-15',dur=1,/noload
;
;Written by:	Matt Davis
;
;-

pro thm_l2gen_fbk, probe=probe, date=date, dur=dur, days=days, hours=hours, minutes=minutes, noload=noload, mastercdf=mastercdf

thm_init

;take a little look at the inputs
;-----------------------------------------------------------------------------

if keyword_set(probe) ne 1 then begin
  print,'You did not select a probe. Please select probe.'
  print,"i.e., thm_l2gen_fbk,probe='a'"
  return
endif

probe_list=['a','b','c','d','e']
index=where(probe eq probe_list)
if index eq -1 then begin
  print, 'You entered invalid probe ID : ',strtrim(probe,2)
  print, "Valid names are : 'a', 'b', 'c', 'd' and 'e'"
  print, "i.e., thm_l2gen_fbk,probe='a'"
  return
endif

case 1 of 
  keyword_set(hours)  	: dur=dur/24.
  keyword_set(minutes)	: dur=dur/1440.
  else			: dur=dur
endcase

sc='th'+probe

if not keyword_set(mastercdf) then mastercdf='mastercdfs/'+sc+'_l2_fbk_00000000_v01.cdf'


;load up all the data we'll be playing with
;-----------------------------------------------------------------------------


timespan,date,dur

if (keyword_set(noload) eq 0) then begin

del_data,'*'   ;  give ourselves a clean slate

;load SST data
thm_load_fbk,probe=probe,lev=1

endif ; data load

;read in master CDF file
;-----------------------------------------------------------------------------

fbk_l2_structure=cdf_load_vars(mastercdf,/all)


;insert data into sst_l2_structure
;-----------------------------------------------------------------------------

fbk_vars=tnames([strjoin(sc+'_fb_*')])
                 

;strip out 'Nan' time records
;-----------------------------------------------------------------------------

for i=0,n_elements(fbk_vars)-1 do begin
  get_data,fbk_vars(i),data=dd
  if (size(dd,/type) eq 8) then index=where(finite(dd.x) eq 1) else index=-1
  if index(0) ne -1 then begin
    str_element,dd,'v',success=success
    if success eq 1 then store_data,fbk_vars(i),data={x:dd.x(index),y:dd.y(index,*),v:dd.v(index,*)} $
      else store_data,fbk_vars(i),data={x:dd.x(index),y:dd.y(index,*)}
  endif
endfor


;insert data into fbk_l2_structure
;-----------------------------------------------------------------------------


mastertags=fbk_l2_structure.vars.name

;data and time
;--------------------------------------------------

for i=0,n_elements(fbk_vars)-1 do begin
  ;data
  get_data,fbk_vars(i),data=dd
  index=where(fbk_vars(i) eq mastertags)
  if size(dd,/type) eq 8 && index ne -1 then fbk_l2_structure.vars[index].dataptr=ptr_new(dd.y)
  ;time
  index=where(strjoin(fbk_vars(i)+'_time') eq mastertags)
  if size(dd,/type) eq 8 && index ne -1 then fbk_l2_structure.vars[index].dataptr=ptr_new(dd.x)
endfor 


;range_epoch

range_epoch_vals=[time_string(date),time_string(time_double(date)+86399.)]

reval_strs=time_string(range_epoch_vals)
year=strmid(reval_strs,0,4)
month=strmid(reval_strs,5,2)
day=strmid(reval_strs,8,2)
hour=strmid(reval_strs,11,2)
minu=strmid(reval_strs,14,2)
sec=strmid(reval_strs,17,2)

cdf_epoch,range_epoch_min,year(0),month(0),day(0),hour(0),minu(0),sec(0),/compute_epoch
cdf_epoch,range_epoch_max,year(1),month(1),day(1),hour(1),minu(1),sec(1),/compute_epoch
range_epoch_values=[range_epoch_min,range_epoch_max]

index=where(strjoin('range_epoch') eq mastertags)
fbk_l2_structure.vars[index].dataptr=ptr_new(range_epoch_values) 

; write sst_l2_structure to L2 CDF file
;-----------------------------------------------------------------------------

filename=strjoin(sc+'_l2_fbk_'+strmid(date,0,4)+strmid(date,5,2)+strmid(date,8,2)+'_v01.cdf')
dummy=cdf_save_vars(fbk_l2_structure,filename)


print, 'L2 FBK CDF file written!   Filename: ',filename 


;clean up
;-----------------------------------------------------------------------------

;unused_ptrs = ptr_extract(fbk_l2_structure)
;ptr_free,unused_ptrs


end