;+
; **********************************************************************
; thm_crib_scm.pro : IDL batch file to get and calibrate SCM data
;  based on batch_calibrate_scm.txt:
; O. Le Contel & P. Robert, CETP, Febrary 11, 2007
; Revised with new IDL_cal_V1.3 software, P. Robert, May 4, 2007
; **********************************************************************
;-

;    ============================
; 1) Select date an time interval
;    ============================

  date = '2007-03-14/00:00:00'
  timespan,date,1,/day

;    =====================
; 2) Select probe and mode
;    =====================

; Select SATNAME (a,b,c,d or e)
  satname = 'd'

; Select MODE (scf, scp, scw)
  mode = 'scp'

;    =====================================================
; 3) get auxiliary data from STAT file and SCM header file
;    =====================================================

; Extract arrays spin_phase, spin_period, spin_direction from STAT file

  scm_get_spin_data, satname, time_spin,val_spinpha, val_spinper, $
                                        val_spinras, val_spindec

; Extract arrays sample_frequency and modes from SCM header file

  scm_get_sample_rate, satname, mode, time_sfreq, val_sfreq, apid_str

;    ============
; 4) get SCM data
;    ============

; get data over a whole day

  thm_load_scm,probe=satname
  thscs_mode = 'th'+satname+'_'+mode
  get_data, thscs_mode, time_waveform, val_waveform, values

; Plot data in a new windows and then select time interval interactively

;;;;  scm_select_from_ctime , thscs_mode, t1, t2

;; To impose by hand t1 and t2 :
  starting_date =strmid(date,0,10)
  starting_time='05:45:06.0'
  ending_time  ='05:47:09.0'
;  starting_time='14:48:13.0'
;  ending_time  ='14:49:09.0'
 
  str_starting = starting_date+'T'+starting_time+'Z'
  str_ending   = starting_date+'T'+ending_time+'Z'
 
  t1 = tu_isodate_to_julsec(str_starting,secofday1)
  t2 = tu_isodate_to_julsec(str_ending,secofday2)

; Reduction of the data arrays to the selected time interval

  scm_reduce_period_V3, 'waveform',        time_waveform, val_waveform, t1, t2

  nw=N_elements(time_waveform)
  t1w=time_waveform(0)
  t2w=time_waveform(nw-1)
  

  scm_reduce_period_S1, 'samp. frequency', time_sfreq   , val_sfreq   , t1w, t2w
  scm_reduce_period_S4, 'spin phase', 'spin period', 'spin rasc', 'spin dec', $
                                         time_spin, val_spinpha,val_spinper, $
                                         val_spinras,val_spindec, t1w, t2w

; Conversion of TM data in Volts

  val_waveform  = val_waveform*(10.04/2.^16)

;    ========================
; 5) create Level 1 data file
;    ========================

; Set the directory for the results, according O.S.

  Dir_result = !themis.local_data_dir+'/SCM_Results/'
  if file_test(Dir_result, /dir) eq 0 then begin
     PRINT, 'Creating new directory: "'+Dir_result+'"'
     file_mkdir, Dir_result
  endif
  Dir_result = file_search(Dir_result)+'/'

  datim= tu_julsec_to_name(time_waveform(0))
  sfreq= STRING(FIX(val_sfreq(0)), FORMAT='(I4.4)')
  nwave=N_elements(time_waveform)
  RFF_name= 'th' +satname +'_' +mode +sfreq +'_L1_' +datim +'.rff'
  PRINT, 'Dir_result= ', Dir_result
  PRINT, 'RFF_name  = ', RFF_name
  PRINT, 'Number of point to be written :', nwave,'  = 3 *',nwave/3
  
  scm_create_rff,Dir_result,RFF_name,$
                 time_waveform,val_waveform,satname,apid_str,$
                 time_spin, val_spinpha, val_spinper,val_spinras,$
                 val_spindec,time_sfreq,val_sfreq

;    ========================
; 6) SCM waveform calibration
;    ========================

; location of the 'cal' directory within the distribution

  dirscm = routine_info('scm_create_rff',/source)
  dirscm = file_dirname(dirscm.path)
  dircal= filepath(root_dir=dirscm,'cal')
  PRINT, 'dirscm: ',dirscm
  PRINT, 'dircal: ',dircal

; define calibration parameters

; Example: Nbp= 256 Despin= 1 Fdet= 0.5 Fcut= 0.1 Fmin= 0. Fmax= 0. step= 4

  Nbp  = 512
  Despin=1
  Fdet = 2.
  Fcut = 0.1
  Fmin = 0.
  Fmax = 0.
  step = 5

; create cowave.in file

  scm_define_calparam, Dir_result, RFF_name,dircal,T1,T2, $
                       Nbp,Despin,Fdet,Fcut,Fmin,Fmax,step

; name of the execuatable file in the distribution, according O.S.
  os= !version.os

  if(os eq 'Win32')      then cowave_exe = '\PCW_V1p3_gfo_cowave.exe' $
  else if(os eq 'linux') then cowave_exe = '/LIN_V1p3_ifo_cowave.exe' $
  else if(os eq 'sunos') then cowave_exe = '/SOL_V1p3_f90_cowave.exe' $
  else if(os eq 'darwin') then cowave_exe = '/MAC_V1p3_gfo_cowave.exe'

  cowave_exe = dirscm +cowave_exe
 
; Running calibration program
; create cowave.resu file containing calibrated waveforms
 
  CD, Dir_result
  command=cowave_exe +' < ' +Dir_result +'cowave.in' $
                          +' > ' +Dir_result +'cowave.out' 

  PRINT, 'spawning ',command
  PRINT, 'Please wait...'
  spawn, command, com_out, /STDERR, COUNT=com_out_dim
  PRINT, com_out, FORMAT='(a,/)'

; Now cowave.resu contains calibrated waveforms, and cowave.out contains
; an execution report.

;    =================================
; 7) get SCM calibrated waveform in nT
;    =================================

  scm_read_cowave, times, Bx,By,Bz, Nbwin,SCname,SCnum,Nbp, $
                   Fsamp,Fspin,Fdet,Fcut, Fmin,Fmax, Step,mode, $
                   title,subtitle,anomaly

; Store all data windows into a single array

  PRINT, 'Store all windows in a single array, and plot the calibrated data'
  
  time_1D = dblarr(Nbwin*Nbp)
  Bxyz = fltarr(Nbwin*Nbp,3)

  FOR isq=0L,Nbwin-1 DO $
      time_1D(isq*Nbp:(isq+1)*Nbp-1)=times(isq)+DINDGEN(Nbp)/Fsamp

  Bxyz(*,0) = transpose(Bx(*,*))
  Bxyz(*,1) = transpose(By(*,*))
  Bxyz(*,2) = transpose(Bz(*,*))

;    =========================================
; 8) Plot calibrated data, with time in second
;    =========================================

  datim1= tu_time_to_str7(time_waveform(0))
  datim2= tu_time_to_str7(times(0))
  datim3= tu_time_to_str7(time_1D(0))
  
  PRINT
  PRINT, 'Rounded date & time in RFF name       = ',datim
  PRINT, 'Time of the 1st point of the waveform = ',datim1
  PRINT, 'Time of the 1st point of time array N = ',datim2
  PRINT, 'Time of the 1st point of time array 1D= ',datim3

  t1sec= DOUBLE(LONG(time_waveform(0)))
  t1str= tu_time_to_str7(t1sec)
  PRINT, 'Plot will start at a rounded second from T1'
  PRINT, 'So t1 rounded= ',t1sec, '    or ',t1str, FORMAT='(a,f16.3,2a)'

  str_Nbp   = string(Nbp   ,format='(i6)')
  str_Despin= string(Despin,format='(i1)')
  str_Fdet  = string(Fdet  ,format='(f7.2)')
  str_Fcut  = string(Fcut  ,format='(f6.2)')
  str_Fmin  = string(Fmin  ,format='(f7.2)')
  str_Fmax  = string(Fmax  ,format='(f7.2)')
  str_step  = string(step  ,format='(i1)')
  str_Fsamp = string(Fsamp ,format='(f5.0)')

  y2=max([Bxyz(*,0),Bxyz(*,1),Bxyz(*,2)], min=y1)

  PLOT, time_1D -t1sec, Bxyz(*,0), YRANGE=[y1,y2], YMARGIN=[5.,3.], $
        TITLE='SCM calibrated data ' +datim, $
        SUBTITLE= 'Nbp='+str_Nbp+ $
              ', step='+str_step+' ,Despin='+str_Despin+$
              ', Fdet='+str_Fdet+'Hz, Fcut='+str_Fcut+'Hz, Fmin=' $
               +str_Fmin+ 'Hz, Fmax='+str_Fmax+'Hz', $
        XTITLE='Second',$
        YTITLE='th'+satname+' '+mode+' '+str_Fsamp+' (nT)', $
        charsize=1.2, /NODATA

  OPLOT, time_1D -t1sec, Bxyz(*,0), color=[ 10]
  OPLOT, time_1D -t1sec, Bxyz(*,1), color=[150]
  OPLOT, time_1D -t1sec, Bxyz(*,2), color=[250]

  
  XYOUTS, 0.12, 0.13,'Bx', color=[ 10], charsize=1.3, /NORMAL
  XYOUTS, 0.16, 0.13,'By', color=[150], charsize=1.3, /NORMAL
  XYOUTS, 0.20, 0.13,'Bz', color=[250], charsize=1.3, /NORMAL
  
; store data for further Themis routines
        
  thscs_mode_volt = 'th'+satname+'_'+mode+'_volt'
  store_data,thscs_mode_volt,data={x:time_waveform,y:val_waveform}
 
  thscs_cal_mode = 'th'+satname+'_cal_'+mode+'_ftn'
  store_data,thscs_cal_mode,data={x:time_1D,y:Bxyz}

  PRINT, 'Ready for other processing'
  PRINT, 'End of thm_crib_scm.pro batch file'
  PRINT, '-----------------------------------------'

; ********************************************
; end of thm_crib_scm.pro
; ********************************************

end
