;+
;Procedure: THM_CAL_EFI
;
;Purpose:  Converts raw EFI (V, EDC, and EAC waveform) data into physical quantities.
;keywords:
;   /VERBOSE or VERBOSE=n ; set to enable diagnostic message output.
;		higher values of n produce more and lower-level diagnostic messages.
;   /ALL
;
;Example:
;   thm_cal_efi, /all
;
;Notes:
;	-- fixed, nominal calibration pars used (gains and frequency responses), rather than proper time-dependent parameters.
;
; $LastChangedBy: kenb-mac $
; $LastChangedDate: 2007-07-03 09:10:02 -0700 (Tue, 03 Jul 2007) $
; $LastChangedRevision: 987 $
; $URL $
;-
pro thm_cal_efi, probe=probe, datatype=datatype, files=files,trange=trange, $
	valid_names=valid_names, verbose=verbose, in_suf=in_suf,out_suf=out_suf

thm_init

efi_valid_names = [ 'vaf', 'vap', 'vaw', 'vbf', 'vbp', 'vbw', 'eff', 'efp', 'efw']	; note that efi_dq is deleted.
if not keyword_set(in_suf) then in_suf=''
if not keyword_set(out_suf) then out_suf=''
if arg_present( valid_names) then begin
    valid_names = efi_valid_names
	message, /info, string( strjoin( efi_valid_names, ','), format='( "Valid names:",X,A,".")')
	return
endif

if n_params() eq 0 then begin
   vprobes = ['a','b','c','d','e']

   if not keyword_set(probe) then probes = vprobes $
   else probes = thm_check_valid_name(strlowcase(probe), vprobes, /include_all)
   if not keyword_set(probes) then return

  dts = thm_check_valid_name(strlowcase(datatype), efi_valid_names, $
                                         /include_all)
   if not keyword_set(dts) then return
endif


for s=0L,n_elements(probes)-1L do begin
     sc = probes[s]
     for n=0L, n_elements(dts)-1L do begin
        name = dts[ n]
		tplot_var_raw = thm_tplot_var( sc, name)+in_suf
		tplot_var = thm_tplot_var( sc, name)+out_suf
                get_data, tplot_var_raw, data=d, limit=l, dlim=dl

		if keyword_set( verbose) then $
			message, /info, string( tplot_var, format='("working on TPLOT variable",X,A)')

		; check to see if the data has already been calibrated; do no further processing if that's the case.
		if( thm_data_calibrated( dl)) then begin
			if keyword_set( verbose) then begin
				message, /info, string( tplot_var, format='(A,X,"has already been calibrated.")')
			endif
			continue	; proceed to the next step of the inner loop over data names.
		endif
		; otherwise, proceed with the RAW->PHYS transformation.

		tplot_var_hed = tplot_var + '_hed'
		get_data, tplot_var_hed, data=d_hed, limit=l_hed, dlim=dl_hed

		; check that returned data and hed structures are structures (get_data returns 0 if no TPLOT variable exists).
		if (size( d, /type) eq 8) and (size( d_hed, /type) eq 8) then begin

			hed_data = thm_unpack_hed( name, d_hed.y)
			switch strlowcase( name) of
				'vaf':
				'vbf':
				'vap':
				'vbp':
				'vaw':
				'vbw':	begin
					tbeg = min( d.x, max=tend)
					thm_get_efi_cal_pars, tbeg, tend, name, cal_pars=cp
					;stop
					res = fltarr( size( d.y, /dimensions))
					for icomp=0L,5L do begin
						res[ *, icomp] = cp.gain[ icomp]*(float(d.y[ *, icomp]) - cp.offset[ icomp])
					endfor
					;stop

                                ; update the DLIMIT elements to reflect RAW->PHYS transformation, coordinate system, etc.

					units = cp.units

                                        str_element, dl, 'data_att', data_att, success=has_data_att
                                        if has_data_att then begin
                                           str_element, data_att, 'data_type', 'calibrated', /add
                                        endif else data_att = { data_type: 'calibrated' }
                                        str_element, data_att, 'coord_sys',  'efi_sensor', /add
                                        str_element, data_att, 'units', units, /add
                                        str_element, data_att, 'cal_par_time', cp.cal_par_time, /add
                                        str_element, dl, 'data_att', data_att, /add

					str_element, dl, 'ytitle', string( tplot_var, units, format='(A,"!C!C[",A,"]")'), /add
					str_element, dl, 'units', units, /add
					str_element, dl, 'labels', [ 'V1', 'V2', 'V3', 'V4', 'V5', 'V6'], /add
					str_element, dl, 'labflag', 1, /add
					str_element, dl, 'colors', [ 1, 2, 3, 4, 5, 6]

					; store the transformed spectra back into the original TPLOT variable.
					store_data, tplot_var, $
						data = { x:d.x, y:res, v:d.v }, $
						lim=l, dlim=dl
					break
				end	; end of { VAF, VBF, VAP, VBP, VAW, VBW} calibration clause.
				'eff':
				'efp':
				'efw':	begin	; { EFF, EFP, EFW} calibration clause.
						tbeg = min( d.x, max=tend)
						thm_get_efi_cal_pars, tbeg, tend, name, cal_pars=cp
						;stop
						e12=49.6
                        e34=40.4
                        e56=5.63
						res = fltarr( size( d.y, /dimensions))
						for icomp=0L,2L do begin
						    if icomp eq 0 then exx=e12
						    if icomp eq 1 then exx=e34
						    if icomp eq 2 then exx=e56
							res[ *, icomp] = -1000*cp.gain[ icomp]*(d.y[ *, icomp] - cp.offset[ icomp])/exx
						endfor
						;stop

						; update the DLIMIT elements to reflect RAW->PHYS transformation, coordinate system, etc.
						cp.units='mV/m'
						units = cp.units
                                                str_element, dl, 'data_att', data_att, success=has_data_att
                                                if has_data_att then begin
                                                   str_element, data_att, 'data_type', 'calibrated', /add
                                                endif else data_att = { data_type: 'calibrated' }
                                                str_element, data_att, 'coord_sys',  'efi_sensor', /add
                                                str_element, data_att, 'units', units, /add
                                                str_element, data_att, 'cal_par_time', cp.cal_par_time, /add

                                                str_element, dl, 'data_att', data_att, /add

						str_element, dl, 'ytitle', string( tplot_var, units, format='(A,"!C!C[",A,"]")'), /add
;						str_element, dl, 'units', units, /add
						str_element, dl, 'labels', [ 'E12', 'E34', 'E56'], /add
						str_element, dl, 'labflag', 1, /add
						str_element, dl, 'colors', [ 2, 4,6]
;						str_element, dl, 'cal_par_time', cp.cal_par_time, /add

						; store the transformed spectra back into the original TPLOT variable.
						store_data, tplot_var, $
							data = { x:d.x, y:res, v:d.v }, $
							lim=l, dlim=dl

					break
				end	; of { EFF, EFP, EFW} calibration clause.
				else:	begin	; improperly defined name of EFI quantity.
				end
			endswitch
		endif else begin	; necessary TPLOT variables not present.
			if keyword_set(verbose) then $
				message, /info, $
					string( tplot_var, tplot_var_hed, $
					format='("necessary TPLOT variables (",A,X,A,") not present for RAW->PHYS transformation.")')
		endelse

	endfor	; loop over names.
endfor	; loop over spacecraft.

end
