;+
;Procedure: THM_LOAD_EFI
;
;Purpose:  Loads THEMIS EFI data
;
;keywords:
;  probe = Probe name. The default is 'all', i.e., load all available probes.
;          This can be an array of strings, e.g., ['a', 'b'] or a
;          single string delimited by spaces, e.g., 'a b'
;  datatype = The type of data to be loaded, can be an array of strings
;          or single string separate by spaces.  The default is 'all'
;  TRANGE= (Optional) Time range of interest  (2 element array), if
;          this is not set, the default is to prompt the user. Note
;          that if the input time range is not a full day, a full
;          day's data is loaded
;  level = the level of the data, the default is 'l1', or level-1
;          data. A string (e.g., 'l2') or an integer can be used. 'all'
;          can be passed in also, to get all levels.
;  CDF_DATA: named variable in which to return cdf data structure: only works
;          for a single spacecraft and datafile name.
;  VARNAMES: names of variables to load from cdf: default is all.
;  /GET_SUPPORT_DATA: load support_data variables as well as data variables
;                      into tplot variables.
;  /DOWNLOADONLY: download file but don't read it.
;  /valid_names, if set, then this routine will return the valid probe, datatype
;          and/or level options in named variables supplied as
;          arguments to the corresponding keywords.
;  files   named varible for output of pathnames of local files.
;  /VERBOSE  set to output some useful info
;  /no_download: use only files which are online locally.
;  relpathnames_all: named variable in which to return all files that are
;          required for specified timespan, probe, datatype, and level.
;          If present, no files will be downloaded, and no data will be loaded.
;  type: 'calibrated' or 'raw'
;  coord: what coordinate system you would like your data in.
;Example:
;   thg_load_efi,/get_suppport_data,probe=['a', 'b']
;Notes:
;
; $LastChangedBy: kramer $
; $LastChangedDate: 2007-07-09 16:16:39 -0700 (Mon, 09 Jul 2007) $
; $LastChangedRevision: 1042 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/branches/QA/idl/themis/spacecraft/fields/thm_load_efi.pro $
;-

pro thm_load_efi,probe=probe, datatype=datatype, trange=trange, $
                 level=level, verbose=verbose, downloadonly=downloadonly, $
                 no_download=no_download,type=type, coord=coord,varformat=varformat,$
                 cdf_data=cdf_data,get_support_data=get_support_data, $
                 varnames=varnames, valid_names = valid_names, files=files, $
                 relpathnames_all = relpathnames_all,$
                 suffix=suffix, progobj=progobj

  thm_init
; If verbose keyword is defined, override !themis.verbose
  vb = size(verbose, /type) ne 0 ? verbose : !themis.verbose
  if not keyword_set(suffix) then suffix=''
  if arg_present(relpathnames_all) then begin
     downloadonly=1
     no_download=1
  end
  vlevels='l1 l2'
  vlevels = strsplit(vlevels, ' ', /extract)
  vprobes=['a','b','c','d','e']
  vdatatypes=['vaf','vap,','vaw','vbf','vbp','vbw','eff','efp','efw', 'eff_0', 'efp_0', 'efw_0']
  if keyword_set(valid_names) then begin

      probe = vprobes
      datatypes = vdatatypes
      return
endif
if not keyword_set(probe) then probes = vprobes $
   else probes = thm_check_valid_name(strlowcase(probe), vprobes, /include_all)
   if not keyword_set(probes) then return

if not keyword_set(datatype) then datatype=vdatatypes
  dts = thm_check_valid_name(strlowcase(datatype), vdatatypes, $
                                         /include_all)
   if not keyword_set(dts) then return

; parse out data level

   if keyword_set(deflevel) then lvl = deflevel else lvl = 'l2'
   if n_elements(level) gt 0 then begin
     if size(level, /type) Eq 7 then begin
       If(level[0] Ne '') Then lvl = strcompress(strlowcase(level), / remove_all)
     endif else lvl = 'l'+strcompress(string(fix(level)), /remove_all)
   endif
   lvls = thm_check_valid_name(strlowcase(lvl), vlevels)
   if not keyword_set(lvls) then return
   if n_elements(lvls) gt 1 then begin
      message, /info, level = -1, $
               'only one value may be specified for level'
      return
   endif
     if lvl eq 'l2' and keyword_set(type) then begin
    print,"Type keyword not valid for level 2 data."
    return
  endif
if not keyword_set(type) then type='calibrated'





 if lvls eq 'l1' then begin
     ;; default action for loading level 1 is to calibrate
     if ~keyword_set(type) || strmatch(type, 'calibrated') then begin
        ;; we're calibrating, so make sure we get support data
        if not keyword_set(get_support_data) then begin
           get_support_data = 1
           delete_support_data = 1
        endif
     endif
  endif
  thm_load_proc_arg, sname=probes, datatype=dts, $
                     level=lvls, verbose=verbose, no_download=no_download, $
                     valid_names = valid_names, $
                     vsnames = 'a b c d e', $
                     type_sname = 'probe', $
                     vdatatypes =  'vaf vap vaw vbf vbp vbw eff efp efw eff_0 efp_0 efw_0', $
                     vlevels = 'l1 l2', $
                     deflevel = 'l1', $
                     osname=probes, odt=dts, olvl=lvls, my_themis=my_themis

  nlvls = n_elements(lvls)
  ndts = n_elements(dts)
  nprobes = n_elements(probes)

  if arg_present(cdf_data) && nlvls*ndts*nsnames gt 1 then begin
     print, 'can only get cdf_data for a single datatype'
     return
  endif

;get file names, loop over all snames, levels and datatypes
  for k = 0, nlvls-1 do $
     for j = 0, ndts-1 do $
        for i = 0, nprobes-1 do begin
     probei = probes[i]
     dtj = dts[j]
     lvlk = lvls[k]

     relpath = 'th'+probei+'/'+lvlk+'/'+ dtj+'/'
     prefix = 'th'+probei+'_'+lvlk+'_'+dtj+'_'
     dir = 'YYYY/'
     ending = '_v01.cdf'

     relpathnames = file_dailynames(relpath, prefix, ending, dir=dir, $
                                    trange = trange,addmaster=addmaster)

     if vb ge 7 then print, 'relpathnames : ', relpathnames

     if arg_present(relpathnames_all) then begin
        if i+j+k eq 0 then relpathnames_all = relpathnames else $
           relpathnames_all = [relpathnames_all, relpathnames]
     endif

     ;;download files for this probe, level, and datatype

     ;; my_themis is a copy of !themis, which may have no_download set
     files = file_retrieve(relpathnames, _extra=my_themis, progobj=progobj)

     if keyword_set(downloadonly) then begin
        continue
     endif

     if arg_present(cdf_data) then begin
        cdf_data = cdf_load_vars(files,varnames=varnames, $
                                 verbose=vb,/all)
        return
     endif
 if lvls[0] eq 'l2' $
      && ~keyword_set(varformat) && keyword_set(vL2coord) then begin
      if not keyword_set(coord) then crds = vL2coord $
      else crds = thm_check_valid_name(strlowcase(coord), vL2coord, /include_all)
      if not keyword_set(crds) then return
      if keyword_set(vb) then printdat, crds, /value, varname='Coord'
      varformat = 'th?_' + strjoin(array_cross(dts, crds), '_')
   endif else varformat='th?_??? th?_???_hed'

     if keyword_set(verbose) then print,transpose(['Loading...',files])
     if keyword_set(get_support_data) then begin
     cdf2tplot,file=files,all=all,verbose=vb, varformat=varformat,$
               tplotnames=tplotnames
     endif else cdf2tplot,file=files,all=all,verbose=vb,varformat=varformat,tplotnames=tplotnames
     ;if keyword_set(get_support_data) then begin

     ;if keyword_set(get_support_data)
     if not keyword_set(suffix) then suffix = ''
     ;; add '_raw' (or whatever) suffix to tplot var name
     ;; and add DLIMIT tags to data quantities
     for l=0, n_elements(tplotnames)-1 do begin
        tplot_var = tplotnames[l]
        get_data, tplot_var, data=d_str, limit=l_str, dlimit=dl_str
        if data_type(dl_str) eq 8 && dl_str.cdf.vatt.var_type eq 'data' $
        then begin
           tplot_var_raw = tplot_var+suffix
           data_att = { data_type:'raw', coord_sys:'efi_sensor',units:'ADC'}
           str_element, dl_str, 'data_att', data_att, /add

           case 1 of
              strmatch( strlowcase( tplot_var), 'th'+probei+'v??'): begin
                 colors = [ 1, 2, 3, 4, 5, 6]
                 labels = [ 'V1', 'V2', 'V3', 'V4', 'V5', 'V6']
              end
              strmatch( strlowcase( tplot_var), 'th'+probei+'e??'): begin
                 colors = [ 2, 4, 6]
                 labels = [ 'e12', 'e34', 'e56']
              end
              else:	begin
              end
           endcase
           str_element, dl_str, 'colors', colors, /add
           str_element, dl_str, 'labels', labels, /add
           str_element, dl_str, 'labflag', 1, /add
           str_element, dl_str, 'ytitle', $
                        string( tplot_var_raw, 'ADC', format='(A,"!C!C[",A,"]")'), /add


           store_data, delete=tplot_var
           store_data, tplot_var_raw, data=d_str, limit=l_str, dlimit=dl_str

           etype=strmid(tplot_var,4)
           if where(dts eq etype+'_0') ne -1 then begin
             get_data,'th'+probei+'_'+etype+suffix,data=thx_exx,limit=l,dlim=dl
             thx_xxx_0=thx_exx.y
             thx_xxx_0[*,2]=0
             str_element, dl_str, 'ytitle', $
                        string( tplot_var+'_0', 'ADC', format='(A,"!C!C[",A,"]")'), /add
             store_data,'th'+probei+'_'+etype+'_0',data={x:thx_exx.x,y:thx_xxx_0},limit=l,dlim=dl
           endif
           ;future code for _dot0 implementation
;           if where(dts eq etype+'_dot0') ne -1 then begin
;             get_data,'th'+probei+'_'+etype,data=thx_exx,limit=l,dlim=dl
;             thm_load_fgm,probe=probei,level='l1',coord='ssl'
;             if etype eq 'efp' or etype eq 'efw' then ftype='fgh' else ftype='fgl'
;             ssl2spg,'th'+probei+'_'+ftype,'th'+probei+'_'+ftype+'_spg',/SSL2SPG
;             get_data,'th'+probei+'_'+ftype+'_spg',data=thx_fgx_spg,limit=fl,dlim=fdl
;             get_data,'th'+probei+'_'+etype,data=thx_exx,limit=l,dlim=dl
;           endif
        endif


     endfor

  endfor                    ;end of loop over all probes, levels and datatypes.




  if type eq 'calibrated' then begin
     thm_cal_efi, probe=probes, datatype=dts, in_suf=suffix, out_suf=suffix
  endif

  if vb ge 8 && arg_present(relpathnames_all) then $
     print, 'relpathnames_all: ', relpathnames_all
 if keyword_set(delete_support_data) then begin
if not size(dts, /n_dim) gt 0 then dt = strsplit(dt, ' ', /extract)

for i = 0, n_elements(dt)-1L do begin
if tnames('th'+probe+'_'+dts[i]+'_hed') ne '' then del_data, 'th'+probe+'_'+dts[i]+'_hed'
endfor
endif

end
