; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

PRO thm_spin_interpol, time_state,spinpha,spinper,time_dat,spinpha_int
; ----------------------------------------------------------------------
;+
;NAME:
;  thm_spin_interpol
;Purpose: 
;  Interpolate spin phase to have same time resolution as time_dat
;Inputs:
;    time_state: double precision array: times of data from state file
;       spinpha: spin phase from state file
;       spinper: spin period from state file
;      time_dat: double precision array: times of data points at which
;                interpolates are desired.
;Output:
;   spinpha_int: interpolated spin phase
;
;Notes:
;  This routine differs from thm_interpolate_state in that
;  it takes the weighted average of the phase obtained by extrapolating
;  forwards from the previous state data point and the phase obtained by
;  extrapolating backwards from the following
;  state data point.
;
; P. Robert, CETP, January 2007
; Some optimizations by K. Bromund, SPSystems/NASA/GSFC, May 2007
;-
; ----------------------------------------------------------------------

; we do nothing if the two time arrays are identical

  void = where(time_state - time_dat ne 0, idiff)
  IF (idiff EQ 0 ) THEN BEGIN
     print, '*** thm_spin_interpol: input time arrays identical, nothing done'
     RETURN
  ENDIF

  nsta=N_ELEMENTS(time_state)
  ndat=N_ELEMENTS(time_dat)
  spinpha_int=FLTARR(ndat)
  nwar=0

  PRINT, '   Dimension of state file array = ', nsta
  PRINT, '   Dimension   of   data   array = ', ndat
  PRINT, '   Starting interpolation, please wait ...'
  FLUSH, 0, -1, -2

; interpolation between the two closest times

  j=0L

  FOR i=0L,ndat-1L DO BEGIN

     time_dat_i = time_dat[i]
; look for the 2 closest times before and after
     WHILE  (time_dat_i GT time_state[j+1] || time_dat_i lt time_state[j] ) $
        && j LE nsta-2  DO BEGIN 
        j=j+1L
     ENDWHILE

     IF j GE nsta-1 THEN BEGIN
        PRINT, '*** thm_spin_interpol: error j+1=',j+1, $
               ' > ndmax=',nsta-1
        PRINT, '*** Nothing is done...'
        RETURN
     ENDIF

; computation of the interpolated value
; two estimate, from time_stae before and time_state after

     tdifA = time_dat_i- time_state[j]
     tdifB = time_state[j+1] - time_dat_i
     phiA= spinpha[j]   +360.D*(tdifA)/spinper[j]
     phiB= spinpha[j+1] -360.D*(tdifB)/spinper[j+1]

     phiA MOD= (360.D)
     phiB MOD= (360.D)

     IF(phiA LT 0.) THEN phiA += 360.
     IF(phiB LT 0.) THEN phiB += 360.

; difference between the two estimate 

     difpha = phiA - phiB
     
; make sure that difpha is the shortest angle between phiA and phiB.
; If not, fix it, and also adjust phiA or phiB so that the 
; weighted average of the two will give the correct angle.
     if difpha gt 180 then begin 
        print, phiA, phiB, i
        difpha -= 360.
        phiB += 360.
        print, phiA, phiB, i
     endif
     if difpha lt -180 then begin
        print, phiA, phiB, i
        difpha += 360.
        phiA += 360.
        print, phiA, phiB, i
     endif

; warning if diff > 2 degres

     IF(ABS(difpha) GT 2.) THEN BEGIN
        nwar=nwar+1
        IF(nwar LT 100) THEN PRINT, $
           '*** thm_spin_interpol: bad estimate, prec.=',difpha,' degres !!!' 
        IF(nwar EQ 100) THEN PRINT, $
           '*** thm_spin_interpol: print warning limited to 100 ...'
     ENDIF

; averaging with different weight

     spinpha_int[i]=  (tdifB*phiA + tdifA*phiB)/(tdifA+tdifB)

;  may still need to normalize phase, because un-normalized phase
;  may necessarily have been used to perform weighted average.
     if spinpha_int[i] gt 360. then begin
        spinpha_int -= 360.
     endif

  ENDFOR

END

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
